# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union

import httpx

from ..types import (
    application_deployment_list_params,
    application_deployment_create_params,
    application_deployment_update_params,
)
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncPageResponse, AsyncPageResponse
from .._base_client import AsyncPaginator, make_request_options
from ..types.application_deployment import ApplicationDeployment

__all__ = ["ApplicationDeploymentsResource", "AsyncApplicationDeploymentsResource"]


class ApplicationDeploymentsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ApplicationDeploymentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return ApplicationDeploymentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ApplicationDeploymentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return ApplicationDeploymentsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        application_variant_id: str,
        endpoint: str,
        is_active: bool,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ApplicationDeployment:
        """
        ### Description

        Creates a application deployment

        ### Details

        This API can be used to create a application deployment. To use this API, review
        the request schema and pass in all fields that are required to create a
        application deployment.

        Args:
          account_id: The ID of the account that owns the given entity.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v4/application-deployments",
            body=maybe_transform(
                {
                    "account_id": account_id,
                    "application_variant_id": application_variant_id,
                    "endpoint": endpoint,
                    "is_active": is_active,
                    "name": name,
                },
                application_deployment_create_params.ApplicationDeploymentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ApplicationDeployment,
        )

    def retrieve(
        self,
        application_deployment_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ApplicationDeployment:
        """
        ### Description

        Gets the details of a application deployment

        ### Details

        This API can be used to get information about a single application deployment by
        ID. To use this API, pass in the `id` that was returned from your Create
        Application Deployment API call as a path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_deployment_id:
            raise ValueError(
                f"Expected a non-empty value for `application_deployment_id` but received {application_deployment_id!r}"
            )
        return self._get(
            f"/v4/application-deployments/{application_deployment_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ApplicationDeployment,
        )

    def update(
        self,
        application_deployment_id: str,
        *,
        is_active: bool | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ApplicationDeployment:
        """
        ### Description

        Updates a application deployment

        ### Details

        This API can be used to update the application deployment that matches the ID
        that was passed in as a path parameter. To use this API, pass in the `id` that
        was returned from your Create Application Deployment API call as a path
        parameter.

        Review the request schema to see the fields that can be updated.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_deployment_id:
            raise ValueError(
                f"Expected a non-empty value for `application_deployment_id` but received {application_deployment_id!r}"
            )
        return self._patch(
            f"/v4/application-deployments/{application_deployment_id}",
            body=maybe_transform(
                {
                    "is_active": is_active,
                    "name": name,
                },
                application_deployment_update_params.ApplicationDeploymentUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ApplicationDeployment,
        )

    def list(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        application_variant_id: Union[int, str] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPageResponse[ApplicationDeployment]:
        """
        ### Description

        Lists all application deployments accessible to the user.

        ### Details

        This API can be used to list application deployments. If a user has access to
        multiple accounts, all application deployments from all accounts the user is
        associated with will be returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/application-deployments",
            page=SyncPageResponse[ApplicationDeployment],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "application_variant_id": application_variant_id,
                        "limit": limit,
                        "page": page,
                    },
                    application_deployment_list_params.ApplicationDeploymentListParams,
                ),
            ),
            model=ApplicationDeployment,
        )


class AsyncApplicationDeploymentsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncApplicationDeploymentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncApplicationDeploymentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncApplicationDeploymentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncApplicationDeploymentsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        application_variant_id: str,
        endpoint: str,
        is_active: bool,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ApplicationDeployment:
        """
        ### Description

        Creates a application deployment

        ### Details

        This API can be used to create a application deployment. To use this API, review
        the request schema and pass in all fields that are required to create a
        application deployment.

        Args:
          account_id: The ID of the account that owns the given entity.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v4/application-deployments",
            body=await async_maybe_transform(
                {
                    "account_id": account_id,
                    "application_variant_id": application_variant_id,
                    "endpoint": endpoint,
                    "is_active": is_active,
                    "name": name,
                },
                application_deployment_create_params.ApplicationDeploymentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ApplicationDeployment,
        )

    async def retrieve(
        self,
        application_deployment_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ApplicationDeployment:
        """
        ### Description

        Gets the details of a application deployment

        ### Details

        This API can be used to get information about a single application deployment by
        ID. To use this API, pass in the `id` that was returned from your Create
        Application Deployment API call as a path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_deployment_id:
            raise ValueError(
                f"Expected a non-empty value for `application_deployment_id` but received {application_deployment_id!r}"
            )
        return await self._get(
            f"/v4/application-deployments/{application_deployment_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ApplicationDeployment,
        )

    async def update(
        self,
        application_deployment_id: str,
        *,
        is_active: bool | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ApplicationDeployment:
        """
        ### Description

        Updates a application deployment

        ### Details

        This API can be used to update the application deployment that matches the ID
        that was passed in as a path parameter. To use this API, pass in the `id` that
        was returned from your Create Application Deployment API call as a path
        parameter.

        Review the request schema to see the fields that can be updated.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_deployment_id:
            raise ValueError(
                f"Expected a non-empty value for `application_deployment_id` but received {application_deployment_id!r}"
            )
        return await self._patch(
            f"/v4/application-deployments/{application_deployment_id}",
            body=await async_maybe_transform(
                {
                    "is_active": is_active,
                    "name": name,
                },
                application_deployment_update_params.ApplicationDeploymentUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ApplicationDeployment,
        )

    def list(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        application_variant_id: Union[int, str] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[ApplicationDeployment, AsyncPageResponse[ApplicationDeployment]]:
        """
        ### Description

        Lists all application deployments accessible to the user.

        ### Details

        This API can be used to list application deployments. If a user has access to
        multiple accounts, all application deployments from all accounts the user is
        associated with will be returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/application-deployments",
            page=AsyncPageResponse[ApplicationDeployment],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "application_variant_id": application_variant_id,
                        "limit": limit,
                        "page": page,
                    },
                    application_deployment_list_params.ApplicationDeploymentListParams,
                ),
            ),
            model=ApplicationDeployment,
        )


class ApplicationDeploymentsResourceWithRawResponse:
    def __init__(self, application_deployments: ApplicationDeploymentsResource) -> None:
        self._application_deployments = application_deployments

        self.create = to_raw_response_wrapper(
            application_deployments.create,
        )
        self.retrieve = to_raw_response_wrapper(
            application_deployments.retrieve,
        )
        self.update = to_raw_response_wrapper(
            application_deployments.update,
        )
        self.list = to_raw_response_wrapper(
            application_deployments.list,
        )


class AsyncApplicationDeploymentsResourceWithRawResponse:
    def __init__(self, application_deployments: AsyncApplicationDeploymentsResource) -> None:
        self._application_deployments = application_deployments

        self.create = async_to_raw_response_wrapper(
            application_deployments.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            application_deployments.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            application_deployments.update,
        )
        self.list = async_to_raw_response_wrapper(
            application_deployments.list,
        )


class ApplicationDeploymentsResourceWithStreamingResponse:
    def __init__(self, application_deployments: ApplicationDeploymentsResource) -> None:
        self._application_deployments = application_deployments

        self.create = to_streamed_response_wrapper(
            application_deployments.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            application_deployments.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            application_deployments.update,
        )
        self.list = to_streamed_response_wrapper(
            application_deployments.list,
        )


class AsyncApplicationDeploymentsResourceWithStreamingResponse:
    def __init__(self, application_deployments: AsyncApplicationDeploymentsResource) -> None:
        self._application_deployments = application_deployments

        self.create = async_to_streamed_response_wrapper(
            application_deployments.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            application_deployments.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            application_deployments.update,
        )
        self.list = async_to_streamed_response_wrapper(
            application_deployments.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Iterable
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from .._utils import PropertyInfo

__all__ = [
    "AgentExecuteParams",
    "Message",
    "MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessage",
    "MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1",
    "MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsTextUserMessageContentParts",
    "MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsImageURLUserMessageContentParts",
    "MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsImageURLUserMessageContentPartsImageURL",
    "MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsImageDataUserMessageContentParts",
    "MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsImageDataUserMessageContentPartsImageData",
    "MessageToolMessage",
    "MessageAgentMessage",
    "MessageAgentMessageToolRequest",
    "MessageEgpAPIBackendServerAPIModelsEgpModelsSystemMessage",
    "Tool",
    "ToolArguments",
    "ToolArgumentsProperties",
    "MemoryStrategy",
    "MemoryStrategyParams",
    "ModelParameters",
]


class AgentExecuteParams(TypedDict, total=False):
    messages: Required[Iterable[Message]]
    """The list of messages in the conversation.

    Expand each message type to see how it works and when to use it. Most
    conversations should begin with a single `user` message.
    """

    model: Required[
        Literal[
            "gpt-4",
            "gpt-4-0613",
            "gpt-4-32k",
            "gpt-4-32k-0613",
            "gpt-3.5-turbo",
            "gpt-3.5-turbo-0613",
            "gpt-3.5-turbo-16k",
            "gpt-3.5-turbo-16k-0613",
            "claude-instant-1",
            "claude-instant-1.1",
            "claude-2",
            "claude-2.0",
        ]
    ]
    """The ID of the model to use for the agent.

    We only support the models listed here so far.
    """

    tools: Required[Iterable[Tool]]
    """The list of specs of tools that the agent can use.

    Each spec must contain a `name` key set to the name of the tool, a `description`
    key set to the description of the tool, and an `arguments` key set to a JSON
    Schema compliant object describing the tool arguments.

    The name and description of each tool is used by the agent to decide when to use
    certain tools. Because some queries are complex and may require multiple tools
    to complete, it is important to make these descriptions as informative as
    possible. If a tool is not being chosen when it should, it is common practice to
    tune the description of the tool to make it more apparent to the agent when the
    tool can be used effectively.
    """

    instructions: str
    """The initial instructions to provide to the agent.

    Use this to guide the agent to act in more specific ways. For example, if you
    have specific rules you want to restrict the agent to follow you can specify
    them here. For example, if I want the agent to always use certain tools before
    others, I can write that rule in these instructions.

    Good prompt engineering is crucial to getting performant results from the agent.
    If you are having trouble getting the agent to perform well, try writing more
    specific instructions here before trying more expensive techniques such as
    swapping in other models or finetuning the underlying LLM.
    """

    memory_strategy: MemoryStrategy
    """The memory strategy to use for the agent.

    A memory strategy is a way to prevent the underlying LLM's context limit from
    being exceeded. Each memory strategy uses a different technique to condense the
    input message list into a smaller payload for the underlying LLM.

    We only support the Last K memory strategy right now, but will be adding new
    strategies soon.
    """

    model_parameters: ModelParameters
    """
    Configuration parameters for the agent model, such as temperature, max_tokens,
    and stop_sequences.

    If not specified, the default value are:

    - temperature: 0.2
    - max_tokens: None (limited by the model's max tokens)
    - stop_sequences: None
    """


class MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsTextUserMessageContentParts(
    TypedDict, total=False
):
    text: Required[str]

    type: Literal["text"]


class MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsImageURLUserMessageContentPartsImageURL(
    TypedDict, total=False
):
    url: Required[str]
    """The URL of the image. Note: only OpenAI supports this."""

    detail: Literal["low", "high", "auto"]
    """Only used for OpenAI. Corresponds to OpenAI's image detail parameter."""


class MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsImageURLUserMessageContentParts(
    TypedDict, total=False
):
    image_url: Required[
        MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsImageURLUserMessageContentPartsImageURL
    ]
    """Specifies the image URL and level of detail. Only supported by OpenAI models"""

    type: Literal["image_url"]


class MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsImageDataUserMessageContentPartsImageData(
    TypedDict, total=False
):
    data: Required[str]
    """The base64-encoded image data."""

    media_type: Required[str]
    """The media/mime type of the image data.

    For example, 'image/png'. Check providers' documentation for supported media
    types.
    """

    detail: Literal["low", "high", "auto"]
    """Only used for OpenAI. Corresponds to OpenAI's image detail parameter."""

    type: Literal["base64"]
    """The type of the image data. Only base64 is supported."""


class MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsImageDataUserMessageContentParts(
    TypedDict, total=False
):
    image_data: Required[
        MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsImageDataUserMessageContentPartsImageData
    ]
    """Specifies inline image data"""

    type: Literal["image_data"]


MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1: TypeAlias = Union[
    MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsTextUserMessageContentParts,
    MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsImageURLUserMessageContentParts,
    MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsImageDataUserMessageContentParts,
]


class MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessage(TypedDict, total=False):
    content: Required[Union[str, Iterable[MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1]]]
    """Input from the user.

    Can either be text or a list of content parts. Not all models support image
    content parts, or multiple parts.
    """

    role: Literal["user"]
    """The role of the message. Must be set to 'user'.

    A user message is a message from the user to the AI. This should be the message
    used to send end user input to the AI.
    """


class MessageToolMessage(TypedDict, total=False):
    content: Required[str]
    """Text output from the tool"""

    name: Required[str]
    """Name of the tool"""

    role: Literal["tool"]
    """The role of the message. Must be set to 'tool'.

    A tool message is a message the client application uses to send tool output to
    the AI. It should contain the name of the tool and the output from the tool
    encoded as text.
    """


class MessageAgentMessageToolRequest(TypedDict, total=False):
    arguments: Required[str]
    """Arguments to pass to the tool.

    The format must be a JSON Schema-compliant object serialized into a string.
    """

    name: Required[str]
    """Name of the tool that the AI wants the client to use."""


class MessageAgentMessage(TypedDict, total=False):
    content: str
    """Text output of the agent if no more tools are needed."""

    role: Literal["agent"]
    """The role of the message. Must be set to 'agent'.

    An agent message is a message generated by an AI agent. It is different than an
    assistant message in that it can either contain a direct content output or a
    tool request that the client application must handle.
    """

    tool_request: MessageAgentMessageToolRequest
    """The tool request if the agent needs more information."""


class MessageEgpAPIBackendServerAPIModelsEgpModelsSystemMessage(TypedDict, total=False):
    content: Required[str]
    """Text input from the system."""

    role: Literal["system"]
    """The role of the message. Must be set to 'system'.

    A system message is different from other messages in that it does not originate
    from a party engaged in a user/AI conversation. Instead, it is a message that is
    injected by either the application or system to guide the conversation. For
    example, a system message may be used as initial instructions for an AI entity
    or to tell the AI that it did not do something correctly.
    """


Message: TypeAlias = Union[
    MessageEgpAPIBackendServerAPIModelsEgpModelsUserMessage,
    MessageToolMessage,
    MessageAgentMessage,
    MessageEgpAPIBackendServerAPIModelsEgpModelsSystemMessage,
]


class ToolArgumentsProperties(TypedDict, total=False):
    description: Required[str]
    """Description of what the argument is used for.

    This description is used to help the Agent generate sensible arguments for the
    tool. It is very important that this description is succinct, clear, and
    accurate.
    """

    type: Required[Literal["string", "number", "integer", "boolean", "object", "array", "null"]]
    """The argument's type.

    The type is used to help the Agent generate valid arguments for the tool.

    For more information about types, see:
    https://json-schema.org/understanding-json-schema/reference/type.html#type-specific-keywords
    """

    default: str
    """A default value for the argument if unspecified."""

    examples: List[str]
    """Example values demonstrating how the argument should look.

    This can be used to help the agent understand what a valid argument should look
    like.
    """


class ToolArguments(TypedDict, total=False):
    type: Required[Literal["object"]]
    """Type of argument. Currently only "object" is supported"""

    properties: Dict[str, ToolArgumentsProperties]
    """
    An object where each key is the name of a keyword argument and each value is a
    schema used to validate that property. Each schema must have a type and
    description, but can also have a default value and examples.

    For more information on how to define a valid property, visit
    https://json-schema.org/understanding-json-schema/reference/object.html
    """


class Tool(TypedDict, total=False):
    arguments: Required[ToolArguments]
    """An JSON Schema-compliant schema for the tool arguments.

    To describe a function that accepts no parameters, provide the value
    `{"type": ""object", "properties": {}}`.

    For more information on how to define a valid JSON Schema object, visit
    https://json-schema.org/understanding-json-schema/reference/object.html
    """

    description: Required[str]
    """Description of the tool.

    Because some queries are complex and may require multiple tools to complete, it
    is important to make these descriptions as informative as possible. If a tool is
    not being chosen when it should, it is common practice to tune the description
    of the tool to make it more apparent to the agent when the tool can be used
    effectively.
    """

    name: Required[str]
    """Name of the tool.

    A tool is a function that the _client application_ has at its disposal. The tool
    name is the name the client wishes the Agent to use to refer to this function
    when it decides if it wants the user to use the tool or not. Itmust be unique
    amongst the set of tools provided in a single API call.
    """


class MemoryStrategyParams(TypedDict, total=False):
    k: Required[int]
    """The maximum number of previous messages to remember."""


class MemoryStrategy(TypedDict, total=False):
    params: Required[MemoryStrategyParams]
    """Configuration parameters for the memory strategy."""

    name: Literal["last_k"]
    """Name of the memory strategy. Must be `last_k`.

    This strategy truncates the message history to the last `k` messages. It is the
    simplest way to prevent the model's context limit from being exceeded. However,
    this strategy only allows the model to have short term memory. For longer term
    memory, please use one of the other strategies.
    """


class ModelParameters(TypedDict, total=False):
    frequency_penalty: float
    """Penalize tokens based on how much they have already appeared in the text.

    Positive values encourage the model to generate new tokens and negative values
    encourage the model to repeat tokens. Available for models from LLM Engine, and
    OpenAI
    """

    max_tokens: int
    """The maximum number of tokens to generate in the completion.

    The token count of your prompt plus max_tokens cannot exceed the model's context
    length. If not, specified, max_tokens will be determined based on the model
    used: | Model API family | Model API default | EGP applied default | | --- | ---
    | --- | | OpenAI Completions |
    [`16`](https://platform.openai.com/docs/api-reference/completions/create#max_tokens)
    | `context window - prompt size` | | OpenAI Chat Completions |
    [`context window - prompt size`](https://platform.openai.com/docs/api-reference/chat/create#max_tokens)
    | `context window - prompt size` | | LLM Engine |
    [`max_new_tokens`](https://github.com/scaleapi/launch-python-client/blob/207adced1c88c1c2907266fa9dd1f1ff3ec0ea5b/launch/client.py#L2910)
    parameter is required | `100` | | Anthropic Claude 2 |
    [`max_tokens_to_sample`](https://docs.anthropic.com/claude/reference/complete_post)
    parameter is required | `10000` |
    """

    presence_penalty: float
    """Penalize tokens based on if they have already appeared in the text.

    Positive values encourage the model to generate new tokens and negative values
    encourage the model to repeat tokens. Available for models from LLM Engine, and
    OpenAI.
    """

    stop_sequences: List[str]
    """List of up to 4 sequences where the API will stop generating further tokens.

    The returned text will not contain the stop sequence.
    """

    temperature: float
    """What sampling temperature to use, between [0, 1].

    Higher values like 0.8 will make the output more random, while lower values like
    0.2 will make it more focused and deterministic. Setting temperature=0.0 will
    enable fully deterministic (greedy) sampling.
    """

    top_k: Annotated[int, PropertyInfo(alias="topK")]
    """Sample from the k most likely next tokens at each step.

    Lower k focuses on higher probability tokens. Available for models from
    Anthropic, Google, and LLM Engine.
    """

    top_p: Annotated[float, PropertyInfo(alias="topP")]
    """The cumulative probability cutoff for token selection.

    Lower values mean sampling from a smaller, more top-weighted nucleus. Available
    for models from Anthropic, Google, Mistral, LLM Engine, and OpenAI.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, TypedDict

__all__ = ["KnowledgeBaseRetrieveParams"]


class KnowledgeBaseRetrieveParams(TypedDict, total=False):
    include_artifacts_status: bool
    """Optional query parameter to include a count of artifacts by status"""

    view: List[Literal["Connections", "ArtifactCount"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable

import httpx

from ..types import application_thread_process_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options

__all__ = ["ApplicationThreadsResource", "AsyncApplicationThreadsResource"]


class ApplicationThreadsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ApplicationThreadsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return ApplicationThreadsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ApplicationThreadsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return ApplicationThreadsResourceWithStreamingResponse(self)

    def process(
        self,
        thread_id: str,
        *,
        application_variant_id: str,
        inputs: object,
        history: Iterable[application_thread_process_params.History] | NotGiven = NOT_GIVEN,
        overrides: Dict[str, application_thread_process_params.Overrides] | NotGiven = NOT_GIVEN,
        stream: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Process Deployed Application For Thread

        Args:
          inputs: Input data for the application. You must provide inputs for each input node

          history: History of the application

          overrides: Optional overrides for the application

          stream: Control to have streaming of the endpoint. If the last node before the output is
              a completion node, you can set this to true to get the output as soon as the
              completion node has a token

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_variant_id:
            raise ValueError(
                f"Expected a non-empty value for `application_variant_id` but received {application_variant_id!r}"
            )
        if not thread_id:
            raise ValueError(f"Expected a non-empty value for `thread_id` but received {thread_id!r}")
        return self._post(
            f"/v4/applications/{application_variant_id}/threads/{thread_id}/process",
            body=maybe_transform(
                {
                    "inputs": inputs,
                    "history": history,
                    "overrides": overrides,
                    "stream": stream,
                },
                application_thread_process_params.ApplicationThreadProcessParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncApplicationThreadsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncApplicationThreadsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncApplicationThreadsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncApplicationThreadsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncApplicationThreadsResourceWithStreamingResponse(self)

    async def process(
        self,
        thread_id: str,
        *,
        application_variant_id: str,
        inputs: object,
        history: Iterable[application_thread_process_params.History] | NotGiven = NOT_GIVEN,
        overrides: Dict[str, application_thread_process_params.Overrides] | NotGiven = NOT_GIVEN,
        stream: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Process Deployed Application For Thread

        Args:
          inputs: Input data for the application. You must provide inputs for each input node

          history: History of the application

          overrides: Optional overrides for the application

          stream: Control to have streaming of the endpoint. If the last node before the output is
              a completion node, you can set this to true to get the output as soon as the
              completion node has a token

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_variant_id:
            raise ValueError(
                f"Expected a non-empty value for `application_variant_id` but received {application_variant_id!r}"
            )
        if not thread_id:
            raise ValueError(f"Expected a non-empty value for `thread_id` but received {thread_id!r}")
        return await self._post(
            f"/v4/applications/{application_variant_id}/threads/{thread_id}/process",
            body=await async_maybe_transform(
                {
                    "inputs": inputs,
                    "history": history,
                    "overrides": overrides,
                    "stream": stream,
                },
                application_thread_process_params.ApplicationThreadProcessParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class ApplicationThreadsResourceWithRawResponse:
    def __init__(self, application_threads: ApplicationThreadsResource) -> None:
        self._application_threads = application_threads

        self.process = to_raw_response_wrapper(
            application_threads.process,
        )


class AsyncApplicationThreadsResourceWithRawResponse:
    def __init__(self, application_threads: AsyncApplicationThreadsResource) -> None:
        self._application_threads = application_threads

        self.process = async_to_raw_response_wrapper(
            application_threads.process,
        )


class ApplicationThreadsResourceWithStreamingResponse:
    def __init__(self, application_threads: ApplicationThreadsResource) -> None:
        self._application_threads = application_threads

        self.process = to_streamed_response_wrapper(
            application_threads.process,
        )


class AsyncApplicationThreadsResourceWithStreamingResponse:
    def __init__(self, application_threads: AsyncApplicationThreadsResource) -> None:
        self._application_threads = application_threads

        self.process = async_to_streamed_response_wrapper(
            application_threads.process,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from typing_extensions import Literal, Required, TypedDict

from ..parameter_bindings_param import ParameterBindingsParam

__all__ = ["CompletionCreateParamsBase", "CompletionCreateParamsNonStreaming", "CompletionCreateParamsStreaming"]


class CompletionCreateParamsBase(TypedDict, total=False):
    prompt: Required[str]

    frequency_penalty: float
    """Penalize tokens based on how much they have already appeared in the text.

    Positive values encourage the model to generate new tokens and negative values
    encourage the model to repeat tokens. Available for models provided by LLM
    Engine and OpenAI.
    """

    logprobs: bool
    """Whether to return logprobs. Currently only supported for llmengine chat models."""

    max_tokens: int
    """The maximum number of tokens to generate in the completion.

    The token count of your prompt plus max_tokens cannot exceed the model's context
    length. If not, specified, max_tokens will be determined based on the model
    used: | Model API family | Model API default | EGP applied default | | --- | ---
    | --- | | OpenAI Completions |
    [`16`](https://platform.openai.com/docs/api-reference/completions/create#max_tokens)
    | `context window - prompt size` | | OpenAI Chat Completions |
    [`context window - prompt size`](https://platform.openai.com/docs/api-reference/chat/create#max_tokens)
    | `context window - prompt size` | | LLM Engine |
    [`max_new_tokens`](https://github.com/scaleapi/launch-python-client/blob/207adced1c88c1c2907266fa9dd1f1ff3ec0ea5b/launch/client.py#L2910)
    parameter is required | `100` | | Anthropic Claude 2 |
    [`max_tokens_to_sample`](https://docs.anthropic.com/claude/reference/complete_post)
    parameter is required | `10000` |
    """

    model_request_parameters: ParameterBindingsParam

    presence_penalty: float
    """Penalize tokens based on if they have already appeared in the text.

    Positive values encourage the model to generate new tokens and negative values
    encourage the model to repeat tokens. Available for models provided by LLM
    Engine and OpenAI.
    """

    stop_sequences: List[str]
    """List of up to 4 sequences where the API will stop generating further tokens.

    The returned text will not contain the stop sequence.
    """

    temperature: float
    """What sampling temperature to use, between [0, 2].

    Higher values like 1.8 will make the output more random, while lower values like
    0.2 will make it more focused and deterministic. Setting temperature=0.0 will
    enable fully deterministic (greedy) sampling.NOTE: The temperature parameter
    range for some model is limited to [0, 1] if the given value is above the
    available range, it defaults to the max value.
    """

    top_k: float
    """Sample from the k most likely next tokens at each step.

    Lower k focuses on higher probability tokens. Available for models provided by
    Google and LLM Engine.
    """

    top_logprobs: int
    """Number of top logprobs to return.

    Currently only supported for llmengine chat models.
    """

    top_p: float
    """The cumulative probability cutoff for token selection.

    Lower values mean sampling from a smaller, more top-weighted nucleus. Available
    for models provided by Google, LLM Engine, and OpenAI.
    """


class CompletionCreateParamsNonStreaming(CompletionCreateParamsBase, total=False):
    stream: Literal[False]
    """Flag indicating whether to stream the completion response"""


class CompletionCreateParamsStreaming(CompletionCreateParamsBase):
    stream: Required[Literal[True]]
    """Flag indicating whether to stream the completion response"""


CompletionCreateParams = Union[CompletionCreateParamsNonStreaming, CompletionCreateParamsStreaming]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, Dict, Union, Mapping, cast
from typing_extensions import Self, Literal, override

import httpx

from . import resources, _exceptions
from ._qs import Querystring
from ._types import (
    NOT_GIVEN,
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
)
from ._utils import (
    is_given,
    get_async_library,
)
from ._version import __version__
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import APIStatusError, SGPClientError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)

__all__ = [
    "ENVIRONMENTS",
    "Timeout",
    "Transport",
    "ProxiesTypes",
    "RequestOptions",
    "resources",
    "SGPClient",
    "AsyncSGPClient",
    "Client",
    "AsyncClient",
]

ENVIRONMENTS: Dict[str, str] = {
    "production": "https://api.egp.scale.com",
    "development": "http://127.0.0.1:5003/public",
}


class SGPClient(SyncAPIClient):
    knowledge_bases: resources.KnowledgeBasesResource
    knowledge_base_data_sources: resources.KnowledgeBaseDataSourcesResource
    chunks: resources.ChunksResource
    agents: resources.AgentsResource
    completions: resources.CompletionsResource
    chat_completions: resources.ChatCompletionsResource
    models: resources.ModelsResource
    model_groups: resources.ModelGroupsResource
    users: resources.UsersResource
    accounts: resources.AccountsResource
    question_sets: resources.QuestionSetsResource
    evaluations: resources.EvaluationsResource
    evaluation_configs: resources.EvaluationConfigsResource
    evaluation_datasets: resources.EvaluationDatasetsResource
    studio_projects: resources.StudioProjectsResource
    application_specs: resources.ApplicationSpecsResource
    questions: resources.QuestionsResource
    model_templates: resources.ModelTemplatesResource
    fine_tuning_jobs: resources.FineTuningJobsResource
    training_datasets: resources.TrainingDatasetsResource
    deployment_packages: resources.DeploymentPackagesResource
    application_variants: resources.ApplicationVariantsResource
    application_deployments: resources.ApplicationDeploymentsResource
    application_variant_reports: resources.ApplicationVariantReportsResource
    application_test_case_outputs: resources.ApplicationTestCaseOutputsResource
    application_schemas: resources.ApplicationSchemasResource
    interactions: resources.InteractionsResource
    applications: resources.ApplicationsResource
    application_threads: resources.ApplicationThreadsResource
    chat_threads: resources.ChatThreadsResource
    themes: resources.ThemesResource
    with_raw_response: SGPClientWithRawResponse
    with_streaming_response: SGPClientWithStreamedResponse

    # client options
    api_key: str
    account_id: str | None

    _environment: Literal["production", "development"] | NotGiven

    def __init__(
        self,
        *,
        api_key: str | None = None,
        account_id: str | None = None,
        environment: Literal["production", "development"] | NotGiven = NOT_GIVEN,
        base_url: str | httpx.URL | None | NotGiven = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous SGPClient client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `api_key` from `SGP_API_KEY`
        - `account_id` from `SGP_ACCOUNT_ID`
        """
        if api_key is None:
            api_key = os.environ.get("SGP_API_KEY")
        if api_key is None:
            raise SGPClientError(
                "The api_key client option must be set either by passing api_key to the client or by setting the SGP_API_KEY environment variable"
            )
        self.api_key = api_key

        if account_id is None:
            account_id = os.environ.get("SGP_ACCOUNT_ID")
        self.account_id = account_id

        self._environment = environment

        base_url_env = os.environ.get("SGP_CLIENT_BASE_URL")
        if is_given(base_url) and base_url is not None:
            # cast required because mypy doesn't understand the type narrowing
            base_url = cast("str | httpx.URL", base_url)  # pyright: ignore[reportUnnecessaryCast]
        elif is_given(environment):
            if base_url_env and base_url is not None:
                raise ValueError(
                    "Ambiguous URL; The `SGP_CLIENT_BASE_URL` env var and the `environment` argument are given. If you want to use the environment, you must pass base_url=None",
                )

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc
        elif base_url_env is not None:
            base_url = base_url_env
        else:
            self._environment = environment = "production"

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self._default_stream_cls = Stream

        self.knowledge_bases = resources.KnowledgeBasesResource(self)
        self.knowledge_base_data_sources = resources.KnowledgeBaseDataSourcesResource(self)
        self.chunks = resources.ChunksResource(self)
        self.agents = resources.AgentsResource(self)
        self.completions = resources.CompletionsResource(self)
        self.chat_completions = resources.ChatCompletionsResource(self)
        self.models = resources.ModelsResource(self)
        self.model_groups = resources.ModelGroupsResource(self)
        self.users = resources.UsersResource(self)
        self.accounts = resources.AccountsResource(self)
        self.question_sets = resources.QuestionSetsResource(self)
        self.evaluations = resources.EvaluationsResource(self)
        self.evaluation_configs = resources.EvaluationConfigsResource(self)
        self.evaluation_datasets = resources.EvaluationDatasetsResource(self)
        self.studio_projects = resources.StudioProjectsResource(self)
        self.application_specs = resources.ApplicationSpecsResource(self)
        self.questions = resources.QuestionsResource(self)
        self.model_templates = resources.ModelTemplatesResource(self)
        self.fine_tuning_jobs = resources.FineTuningJobsResource(self)
        self.training_datasets = resources.TrainingDatasetsResource(self)
        self.deployment_packages = resources.DeploymentPackagesResource(self)
        self.application_variants = resources.ApplicationVariantsResource(self)
        self.application_deployments = resources.ApplicationDeploymentsResource(self)
        self.application_variant_reports = resources.ApplicationVariantReportsResource(self)
        self.application_test_case_outputs = resources.ApplicationTestCaseOutputsResource(self)
        self.application_schemas = resources.ApplicationSchemasResource(self)
        self.interactions = resources.InteractionsResource(self)
        self.applications = resources.ApplicationsResource(self)
        self.application_threads = resources.ApplicationThreadsResource(self)
        self.chat_threads = resources.ChatThreadsResource(self)
        self.themes = resources.ThemesResource(self)
        self.with_raw_response = SGPClientWithRawResponse(self)
        self.with_streaming_response = SGPClientWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"x-api-key": api_key}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            "x-selected-account-id": self.account_id if self.account_id is not None else Omit(),
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        account_id: str | None = None,
        environment: Literal["production", "development"] | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = NOT_GIVEN,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = NOT_GIVEN,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            account_id=account_id or self.account_id,
            base_url=base_url or self.base_url,
            environment=environment or self._environment,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncSGPClient(AsyncAPIClient):
    knowledge_bases: resources.AsyncKnowledgeBasesResource
    knowledge_base_data_sources: resources.AsyncKnowledgeBaseDataSourcesResource
    chunks: resources.AsyncChunksResource
    agents: resources.AsyncAgentsResource
    completions: resources.AsyncCompletionsResource
    chat_completions: resources.AsyncChatCompletionsResource
    models: resources.AsyncModelsResource
    model_groups: resources.AsyncModelGroupsResource
    users: resources.AsyncUsersResource
    accounts: resources.AsyncAccountsResource
    question_sets: resources.AsyncQuestionSetsResource
    evaluations: resources.AsyncEvaluationsResource
    evaluation_configs: resources.AsyncEvaluationConfigsResource
    evaluation_datasets: resources.AsyncEvaluationDatasetsResource
    studio_projects: resources.AsyncStudioProjectsResource
    application_specs: resources.AsyncApplicationSpecsResource
    questions: resources.AsyncQuestionsResource
    model_templates: resources.AsyncModelTemplatesResource
    fine_tuning_jobs: resources.AsyncFineTuningJobsResource
    training_datasets: resources.AsyncTrainingDatasetsResource
    deployment_packages: resources.AsyncDeploymentPackagesResource
    application_variants: resources.AsyncApplicationVariantsResource
    application_deployments: resources.AsyncApplicationDeploymentsResource
    application_variant_reports: resources.AsyncApplicationVariantReportsResource
    application_test_case_outputs: resources.AsyncApplicationTestCaseOutputsResource
    application_schemas: resources.AsyncApplicationSchemasResource
    interactions: resources.AsyncInteractionsResource
    applications: resources.AsyncApplicationsResource
    application_threads: resources.AsyncApplicationThreadsResource
    chat_threads: resources.AsyncChatThreadsResource
    themes: resources.AsyncThemesResource
    with_raw_response: AsyncSGPClientWithRawResponse
    with_streaming_response: AsyncSGPClientWithStreamedResponse

    # client options
    api_key: str
    account_id: str | None

    _environment: Literal["production", "development"] | NotGiven

    def __init__(
        self,
        *,
        api_key: str | None = None,
        account_id: str | None = None,
        environment: Literal["production", "development"] | NotGiven = NOT_GIVEN,
        base_url: str | httpx.URL | None | NotGiven = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async SGPClient client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `api_key` from `SGP_API_KEY`
        - `account_id` from `SGP_ACCOUNT_ID`
        """
        if api_key is None:
            api_key = os.environ.get("SGP_API_KEY")
        if api_key is None:
            raise SGPClientError(
                "The api_key client option must be set either by passing api_key to the client or by setting the SGP_API_KEY environment variable"
            )
        self.api_key = api_key

        if account_id is None:
            account_id = os.environ.get("SGP_ACCOUNT_ID")
        self.account_id = account_id

        self._environment = environment

        base_url_env = os.environ.get("SGP_CLIENT_BASE_URL")
        if is_given(base_url) and base_url is not None:
            # cast required because mypy doesn't understand the type narrowing
            base_url = cast("str | httpx.URL", base_url)  # pyright: ignore[reportUnnecessaryCast]
        elif is_given(environment):
            if base_url_env and base_url is not None:
                raise ValueError(
                    "Ambiguous URL; The `SGP_CLIENT_BASE_URL` env var and the `environment` argument are given. If you want to use the environment, you must pass base_url=None",
                )

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc
        elif base_url_env is not None:
            base_url = base_url_env
        else:
            self._environment = environment = "production"

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self._default_stream_cls = AsyncStream

        self.knowledge_bases = resources.AsyncKnowledgeBasesResource(self)
        self.knowledge_base_data_sources = resources.AsyncKnowledgeBaseDataSourcesResource(self)
        self.chunks = resources.AsyncChunksResource(self)
        self.agents = resources.AsyncAgentsResource(self)
        self.completions = resources.AsyncCompletionsResource(self)
        self.chat_completions = resources.AsyncChatCompletionsResource(self)
        self.models = resources.AsyncModelsResource(self)
        self.model_groups = resources.AsyncModelGroupsResource(self)
        self.users = resources.AsyncUsersResource(self)
        self.accounts = resources.AsyncAccountsResource(self)
        self.question_sets = resources.AsyncQuestionSetsResource(self)
        self.evaluations = resources.AsyncEvaluationsResource(self)
        self.evaluation_configs = resources.AsyncEvaluationConfigsResource(self)
        self.evaluation_datasets = resources.AsyncEvaluationDatasetsResource(self)
        self.studio_projects = resources.AsyncStudioProjectsResource(self)
        self.application_specs = resources.AsyncApplicationSpecsResource(self)
        self.questions = resources.AsyncQuestionsResource(self)
        self.model_templates = resources.AsyncModelTemplatesResource(self)
        self.fine_tuning_jobs = resources.AsyncFineTuningJobsResource(self)
        self.training_datasets = resources.AsyncTrainingDatasetsResource(self)
        self.deployment_packages = resources.AsyncDeploymentPackagesResource(self)
        self.application_variants = resources.AsyncApplicationVariantsResource(self)
        self.application_deployments = resources.AsyncApplicationDeploymentsResource(self)
        self.application_variant_reports = resources.AsyncApplicationVariantReportsResource(self)
        self.application_test_case_outputs = resources.AsyncApplicationTestCaseOutputsResource(self)
        self.application_schemas = resources.AsyncApplicationSchemasResource(self)
        self.interactions = resources.AsyncInteractionsResource(self)
        self.applications = resources.AsyncApplicationsResource(self)
        self.application_threads = resources.AsyncApplicationThreadsResource(self)
        self.chat_threads = resources.AsyncChatThreadsResource(self)
        self.themes = resources.AsyncThemesResource(self)
        self.with_raw_response = AsyncSGPClientWithRawResponse(self)
        self.with_streaming_response = AsyncSGPClientWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"x-api-key": api_key}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            "x-selected-account-id": self.account_id if self.account_id is not None else Omit(),
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        account_id: str | None = None,
        environment: Literal["production", "development"] | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = NOT_GIVEN,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = NOT_GIVEN,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            account_id=account_id or self.account_id,
            base_url=base_url or self.base_url,
            environment=environment or self._environment,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class SGPClientWithRawResponse:
    def __init__(self, client: SGPClient) -> None:
        self.knowledge_bases = resources.KnowledgeBasesResourceWithRawResponse(client.knowledge_bases)
        self.knowledge_base_data_sources = resources.KnowledgeBaseDataSourcesResourceWithRawResponse(
            client.knowledge_base_data_sources
        )
        self.chunks = resources.ChunksResourceWithRawResponse(client.chunks)
        self.agents = resources.AgentsResourceWithRawResponse(client.agents)
        self.completions = resources.CompletionsResourceWithRawResponse(client.completions)
        self.chat_completions = resources.ChatCompletionsResourceWithRawResponse(client.chat_completions)
        self.models = resources.ModelsResourceWithRawResponse(client.models)
        self.model_groups = resources.ModelGroupsResourceWithRawResponse(client.model_groups)
        self.users = resources.UsersResourceWithRawResponse(client.users)
        self.accounts = resources.AccountsResourceWithRawResponse(client.accounts)
        self.question_sets = resources.QuestionSetsResourceWithRawResponse(client.question_sets)
        self.evaluations = resources.EvaluationsResourceWithRawResponse(client.evaluations)
        self.evaluation_configs = resources.EvaluationConfigsResourceWithRawResponse(client.evaluation_configs)
        self.evaluation_datasets = resources.EvaluationDatasetsResourceWithRawResponse(client.evaluation_datasets)
        self.studio_projects = resources.StudioProjectsResourceWithRawResponse(client.studio_projects)
        self.application_specs = resources.ApplicationSpecsResourceWithRawResponse(client.application_specs)
        self.questions = resources.QuestionsResourceWithRawResponse(client.questions)
        self.model_templates = resources.ModelTemplatesResourceWithRawResponse(client.model_templates)
        self.fine_tuning_jobs = resources.FineTuningJobsResourceWithRawResponse(client.fine_tuning_jobs)
        self.training_datasets = resources.TrainingDatasetsResourceWithRawResponse(client.training_datasets)
        self.deployment_packages = resources.DeploymentPackagesResourceWithRawResponse(client.deployment_packages)
        self.application_variants = resources.ApplicationVariantsResourceWithRawResponse(client.application_variants)
        self.application_deployments = resources.ApplicationDeploymentsResourceWithRawResponse(
            client.application_deployments
        )
        self.application_variant_reports = resources.ApplicationVariantReportsResourceWithRawResponse(
            client.application_variant_reports
        )
        self.application_test_case_outputs = resources.ApplicationTestCaseOutputsResourceWithRawResponse(
            client.application_test_case_outputs
        )
        self.application_schemas = resources.ApplicationSchemasResourceWithRawResponse(client.application_schemas)
        self.interactions = resources.InteractionsResourceWithRawResponse(client.interactions)
        self.applications = resources.ApplicationsResourceWithRawResponse(client.applications)
        self.application_threads = resources.ApplicationThreadsResourceWithRawResponse(client.application_threads)
        self.chat_threads = resources.ChatThreadsResourceWithRawResponse(client.chat_threads)
        self.themes = resources.ThemesResourceWithRawResponse(client.themes)


class AsyncSGPClientWithRawResponse:
    def __init__(self, client: AsyncSGPClient) -> None:
        self.knowledge_bases = resources.AsyncKnowledgeBasesResourceWithRawResponse(client.knowledge_bases)
        self.knowledge_base_data_sources = resources.AsyncKnowledgeBaseDataSourcesResourceWithRawResponse(
            client.knowledge_base_data_sources
        )
        self.chunks = resources.AsyncChunksResourceWithRawResponse(client.chunks)
        self.agents = resources.AsyncAgentsResourceWithRawResponse(client.agents)
        self.completions = resources.AsyncCompletionsResourceWithRawResponse(client.completions)
        self.chat_completions = resources.AsyncChatCompletionsResourceWithRawResponse(client.chat_completions)
        self.models = resources.AsyncModelsResourceWithRawResponse(client.models)
        self.model_groups = resources.AsyncModelGroupsResourceWithRawResponse(client.model_groups)
        self.users = resources.AsyncUsersResourceWithRawResponse(client.users)
        self.accounts = resources.AsyncAccountsResourceWithRawResponse(client.accounts)
        self.question_sets = resources.AsyncQuestionSetsResourceWithRawResponse(client.question_sets)
        self.evaluations = resources.AsyncEvaluationsResourceWithRawResponse(client.evaluations)
        self.evaluation_configs = resources.AsyncEvaluationConfigsResourceWithRawResponse(client.evaluation_configs)
        self.evaluation_datasets = resources.AsyncEvaluationDatasetsResourceWithRawResponse(client.evaluation_datasets)
        self.studio_projects = resources.AsyncStudioProjectsResourceWithRawResponse(client.studio_projects)
        self.application_specs = resources.AsyncApplicationSpecsResourceWithRawResponse(client.application_specs)
        self.questions = resources.AsyncQuestionsResourceWithRawResponse(client.questions)
        self.model_templates = resources.AsyncModelTemplatesResourceWithRawResponse(client.model_templates)
        self.fine_tuning_jobs = resources.AsyncFineTuningJobsResourceWithRawResponse(client.fine_tuning_jobs)
        self.training_datasets = resources.AsyncTrainingDatasetsResourceWithRawResponse(client.training_datasets)
        self.deployment_packages = resources.AsyncDeploymentPackagesResourceWithRawResponse(client.deployment_packages)
        self.application_variants = resources.AsyncApplicationVariantsResourceWithRawResponse(
            client.application_variants
        )
        self.application_deployments = resources.AsyncApplicationDeploymentsResourceWithRawResponse(
            client.application_deployments
        )
        self.application_variant_reports = resources.AsyncApplicationVariantReportsResourceWithRawResponse(
            client.application_variant_reports
        )
        self.application_test_case_outputs = resources.AsyncApplicationTestCaseOutputsResourceWithRawResponse(
            client.application_test_case_outputs
        )
        self.application_schemas = resources.AsyncApplicationSchemasResourceWithRawResponse(client.application_schemas)
        self.interactions = resources.AsyncInteractionsResourceWithRawResponse(client.interactions)
        self.applications = resources.AsyncApplicationsResourceWithRawResponse(client.applications)
        self.application_threads = resources.AsyncApplicationThreadsResourceWithRawResponse(client.application_threads)
        self.chat_threads = resources.AsyncChatThreadsResourceWithRawResponse(client.chat_threads)
        self.themes = resources.AsyncThemesResourceWithRawResponse(client.themes)


class SGPClientWithStreamedResponse:
    def __init__(self, client: SGPClient) -> None:
        self.knowledge_bases = resources.KnowledgeBasesResourceWithStreamingResponse(client.knowledge_bases)
        self.knowledge_base_data_sources = resources.KnowledgeBaseDataSourcesResourceWithStreamingResponse(
            client.knowledge_base_data_sources
        )
        self.chunks = resources.ChunksResourceWithStreamingResponse(client.chunks)
        self.agents = resources.AgentsResourceWithStreamingResponse(client.agents)
        self.completions = resources.CompletionsResourceWithStreamingResponse(client.completions)
        self.chat_completions = resources.ChatCompletionsResourceWithStreamingResponse(client.chat_completions)
        self.models = resources.ModelsResourceWithStreamingResponse(client.models)
        self.model_groups = resources.ModelGroupsResourceWithStreamingResponse(client.model_groups)
        self.users = resources.UsersResourceWithStreamingResponse(client.users)
        self.accounts = resources.AccountsResourceWithStreamingResponse(client.accounts)
        self.question_sets = resources.QuestionSetsResourceWithStreamingResponse(client.question_sets)
        self.evaluations = resources.EvaluationsResourceWithStreamingResponse(client.evaluations)
        self.evaluation_configs = resources.EvaluationConfigsResourceWithStreamingResponse(client.evaluation_configs)
        self.evaluation_datasets = resources.EvaluationDatasetsResourceWithStreamingResponse(client.evaluation_datasets)
        self.studio_projects = resources.StudioProjectsResourceWithStreamingResponse(client.studio_projects)
        self.application_specs = resources.ApplicationSpecsResourceWithStreamingResponse(client.application_specs)
        self.questions = resources.QuestionsResourceWithStreamingResponse(client.questions)
        self.model_templates = resources.ModelTemplatesResourceWithStreamingResponse(client.model_templates)
        self.fine_tuning_jobs = resources.FineTuningJobsResourceWithStreamingResponse(client.fine_tuning_jobs)
        self.training_datasets = resources.TrainingDatasetsResourceWithStreamingResponse(client.training_datasets)
        self.deployment_packages = resources.DeploymentPackagesResourceWithStreamingResponse(client.deployment_packages)
        self.application_variants = resources.ApplicationVariantsResourceWithStreamingResponse(
            client.application_variants
        )
        self.application_deployments = resources.ApplicationDeploymentsResourceWithStreamingResponse(
            client.application_deployments
        )
        self.application_variant_reports = resources.ApplicationVariantReportsResourceWithStreamingResponse(
            client.application_variant_reports
        )
        self.application_test_case_outputs = resources.ApplicationTestCaseOutputsResourceWithStreamingResponse(
            client.application_test_case_outputs
        )
        self.application_schemas = resources.ApplicationSchemasResourceWithStreamingResponse(client.application_schemas)
        self.interactions = resources.InteractionsResourceWithStreamingResponse(client.interactions)
        self.applications = resources.ApplicationsResourceWithStreamingResponse(client.applications)
        self.application_threads = resources.ApplicationThreadsResourceWithStreamingResponse(client.application_threads)
        self.chat_threads = resources.ChatThreadsResourceWithStreamingResponse(client.chat_threads)
        self.themes = resources.ThemesResourceWithStreamingResponse(client.themes)


class AsyncSGPClientWithStreamedResponse:
    def __init__(self, client: AsyncSGPClient) -> None:
        self.knowledge_bases = resources.AsyncKnowledgeBasesResourceWithStreamingResponse(client.knowledge_bases)
        self.knowledge_base_data_sources = resources.AsyncKnowledgeBaseDataSourcesResourceWithStreamingResponse(
            client.knowledge_base_data_sources
        )
        self.chunks = resources.AsyncChunksResourceWithStreamingResponse(client.chunks)
        self.agents = resources.AsyncAgentsResourceWithStreamingResponse(client.agents)
        self.completions = resources.AsyncCompletionsResourceWithStreamingResponse(client.completions)
        self.chat_completions = resources.AsyncChatCompletionsResourceWithStreamingResponse(client.chat_completions)
        self.models = resources.AsyncModelsResourceWithStreamingResponse(client.models)
        self.model_groups = resources.AsyncModelGroupsResourceWithStreamingResponse(client.model_groups)
        self.users = resources.AsyncUsersResourceWithStreamingResponse(client.users)
        self.accounts = resources.AsyncAccountsResourceWithStreamingResponse(client.accounts)
        self.question_sets = resources.AsyncQuestionSetsResourceWithStreamingResponse(client.question_sets)
        self.evaluations = resources.AsyncEvaluationsResourceWithStreamingResponse(client.evaluations)
        self.evaluation_configs = resources.AsyncEvaluationConfigsResourceWithStreamingResponse(
            client.evaluation_configs
        )
        self.evaluation_datasets = resources.AsyncEvaluationDatasetsResourceWithStreamingResponse(
            client.evaluation_datasets
        )
        self.studio_projects = resources.AsyncStudioProjectsResourceWithStreamingResponse(client.studio_projects)
        self.application_specs = resources.AsyncApplicationSpecsResourceWithStreamingResponse(client.application_specs)
        self.questions = resources.AsyncQuestionsResourceWithStreamingResponse(client.questions)
        self.model_templates = resources.AsyncModelTemplatesResourceWithStreamingResponse(client.model_templates)
        self.fine_tuning_jobs = resources.AsyncFineTuningJobsResourceWithStreamingResponse(client.fine_tuning_jobs)
        self.training_datasets = resources.AsyncTrainingDatasetsResourceWithStreamingResponse(client.training_datasets)
        self.deployment_packages = resources.AsyncDeploymentPackagesResourceWithStreamingResponse(
            client.deployment_packages
        )
        self.application_variants = resources.AsyncApplicationVariantsResourceWithStreamingResponse(
            client.application_variants
        )
        self.application_deployments = resources.AsyncApplicationDeploymentsResourceWithStreamingResponse(
            client.application_deployments
        )
        self.application_variant_reports = resources.AsyncApplicationVariantReportsResourceWithStreamingResponse(
            client.application_variant_reports
        )
        self.application_test_case_outputs = resources.AsyncApplicationTestCaseOutputsResourceWithStreamingResponse(
            client.application_test_case_outputs
        )
        self.application_schemas = resources.AsyncApplicationSchemasResourceWithStreamingResponse(
            client.application_schemas
        )
        self.interactions = resources.AsyncInteractionsResourceWithStreamingResponse(client.interactions)
        self.applications = resources.AsyncApplicationsResourceWithStreamingResponse(client.applications)
        self.application_threads = resources.AsyncApplicationThreadsResourceWithStreamingResponse(
            client.application_threads
        )
        self.chat_threads = resources.AsyncChatThreadsResourceWithStreamingResponse(client.chat_threads)
        self.themes = resources.AsyncThemesResourceWithStreamingResponse(client.themes)


Client = SGPClient

AsyncClient = AsyncSGPClient



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, TypeAlias

from .._compat import PYDANTIC_V2, ConfigDict
from .._models import BaseModel
from .knowledge_base_data_source import KnowledgeBaseDataSource

__all__ = [
    "KnowledgeBase",
    "EmbeddingConfig",
    "EmbeddingConfigEmbeddingConfigModelsAPI",
    "EmbeddingConfigEmbeddingConfigBase",
    "ArtifactsStatus",
    "Connection",
]


class EmbeddingConfigEmbeddingConfigModelsAPI(BaseModel):
    model_deployment_id: str
    """The ID of the deployment of the created model in the Models API V3."""

    type: Literal["models_api"]
    """The type of the embedding configuration."""

    if PYDANTIC_V2:
        # allow fields with a `model_` prefix
        model_config = ConfigDict(protected_namespaces=tuple())


class EmbeddingConfigEmbeddingConfigBase(BaseModel):
    embedding_model: Literal[
        "sentence-transformers/all-MiniLM-L12-v2",
        "sentence-transformers/all-mpnet-base-v2",
        "sentence-transformers/multi-qa-distilbert-cos-v1",
        "sentence-transformers/paraphrase-multilingual-mpnet-base-v2",
        "openai/text-embedding-ada-002",
        "openai/text-embedding-3-small",
        "openai/text-embedding-3-large",
        "embed-english-v3.0",
        "embed-english-light-v3.0",
        "embed-multilingual-v3.0",
    ]
    """An enumeration."""

    type: Optional[Literal["base"]] = None
    """The type of the embedding configuration."""


EmbeddingConfig: TypeAlias = Union[EmbeddingConfigEmbeddingConfigModelsAPI, EmbeddingConfigEmbeddingConfigBase]


class ArtifactsStatus(BaseModel):
    artifacts_chunking: int
    """Number of artifacts in the chunking state"""

    artifacts_completed: int
    """Number of artifacts uploaded successfully."""

    artifacts_embedding: int
    """Number of artifacts in the embedding state"""

    artifacts_failed: int
    """Number of artifacts that failed while being processed."""

    artifacts_pending: int
    """Previously: Number of artifacts awaiting upload.

    Note that this status will be deprecated soon and should show 0
    """

    artifacts_uploading: int
    """Number of artifacts with upload in progress."""


class Connection(BaseModel):
    knowledge_base_data_source: KnowledgeBaseDataSource
    """The knowledge base data source entity."""

    last_uploaded_at: datetime
    """The date and time when the last upload for the data source was initiated."""

    deletion_status: Optional[Literal["DELETING", "FAILED"]] = None
    """An enumeration."""


class KnowledgeBase(BaseModel):
    created_at: str
    """The timestamp at which the knowledge base was created"""

    embedding_config: EmbeddingConfig
    """The embedding configuration"""

    knowledge_base_id: str
    """The unique ID of the knowledge base"""

    knowledge_base_name: str
    """The name of the knowledge base"""

    vector_store_id: str
    """(Legacy) The unique ID of the underlying vector store.

    This is to allow for backwards compatibility with the V1 Vector Store APIs. This
    will be removed in the near future.
    """

    artifact_count: Optional[int] = None
    """The total number of artifacts in the knowledge base.

    Only returned with the `view=ArtifactCount` query parameter.
    """

    artifacts_status: Optional[ArtifactsStatus] = None
    """
    Number of artifacts in each of the various states, such as completed and failed
    for this knowledge base. This includes all data sources.
    """

    cluster_status: Optional[str] = None
    """Whether the knowledge base has been clustered."""

    connections: Optional[List[Connection]] = None
    """The data source connections associated with the knowledge base.

    Only returned with the `view=Connections` query parameter.
    """

    metadata: Optional[object] = None
    """Metadata associated with the knowledge base"""

    updated_at: Optional[str] = None
    """The timestamp at which the knowledge base was last updated"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["TrainingDataset"]


class TrainingDataset(BaseModel):
    id: str
    """The unique identifier of the entity."""

    account_id: str
    """The ID of the account that owns the given entity."""

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    created_by_user_id: str
    """The user who originally created the entity."""

    data_source: Literal[
        "S3", "SharePoint", "LocalFile", "LocalChunks", "GoogleDrive", "AzureBlobStorage", "Confluence", "Slack"
    ]
    """An enumeration."""

    name: str
    """The name of the dataset"""

    schema_type: Literal["GENERATION", "RERANKING_QUESTIONS"]
    """An enumeration."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .chunks import (
    ChunksResource,
    AsyncChunksResource,
    ChunksResourceWithRawResponse,
    AsyncChunksResourceWithRawResponse,
    ChunksResourceWithStreamingResponse,
    AsyncChunksResourceWithStreamingResponse,
)
from .uploads import (
    UploadsResource,
    AsyncUploadsResource,
    UploadsResourceWithRawResponse,
    AsyncUploadsResourceWithRawResponse,
    UploadsResourceWithStreamingResponse,
    AsyncUploadsResourceWithStreamingResponse,
)
from .artifacts import (
    ArtifactsResource,
    AsyncArtifactsResource,
    ArtifactsResourceWithRawResponse,
    AsyncArtifactsResourceWithRawResponse,
    ArtifactsResourceWithStreamingResponse,
    AsyncArtifactsResourceWithStreamingResponse,
)
from .async_jobs import (
    AsyncJobsResource,
    AsyncAsyncJobsResource,
    AsyncJobsResourceWithRawResponse,
    AsyncAsyncJobsResourceWithRawResponse,
    AsyncJobsResourceWithStreamingResponse,
    AsyncAsyncJobsResourceWithStreamingResponse,
)
from .knowledge_bases import (
    KnowledgeBasesResource,
    AsyncKnowledgeBasesResource,
    KnowledgeBasesResourceWithRawResponse,
    AsyncKnowledgeBasesResourceWithRawResponse,
    KnowledgeBasesResourceWithStreamingResponse,
    AsyncKnowledgeBasesResourceWithStreamingResponse,
)
from .upload_schedules import (
    UploadSchedulesResource,
    AsyncUploadSchedulesResource,
    UploadSchedulesResourceWithRawResponse,
    AsyncUploadSchedulesResourceWithRawResponse,
    UploadSchedulesResourceWithStreamingResponse,
    AsyncUploadSchedulesResourceWithStreamingResponse,
)
from .data_source_connections import (
    DataSourceConnectionsResource,
    AsyncDataSourceConnectionsResource,
    DataSourceConnectionsResourceWithRawResponse,
    AsyncDataSourceConnectionsResourceWithRawResponse,
    DataSourceConnectionsResourceWithStreamingResponse,
    AsyncDataSourceConnectionsResourceWithStreamingResponse,
)

__all__ = [
    "AsyncJobsResource",
    "AsyncAsyncJobsResource",
    "AsyncJobsResourceWithRawResponse",
    "AsyncAsyncJobsResourceWithRawResponse",
    "AsyncJobsResourceWithStreamingResponse",
    "AsyncAsyncJobsResourceWithStreamingResponse",
    "ChunksResource",
    "AsyncChunksResource",
    "ChunksResourceWithRawResponse",
    "AsyncChunksResourceWithRawResponse",
    "ChunksResourceWithStreamingResponse",
    "AsyncChunksResourceWithStreamingResponse",
    "DataSourceConnectionsResource",
    "AsyncDataSourceConnectionsResource",
    "DataSourceConnectionsResourceWithRawResponse",
    "AsyncDataSourceConnectionsResourceWithRawResponse",
    "DataSourceConnectionsResourceWithStreamingResponse",
    "AsyncDataSourceConnectionsResourceWithStreamingResponse",
    "UploadsResource",
    "AsyncUploadsResource",
    "UploadsResourceWithRawResponse",
    "AsyncUploadsResourceWithRawResponse",
    "UploadsResourceWithStreamingResponse",
    "AsyncUploadsResourceWithStreamingResponse",
    "ArtifactsResource",
    "AsyncArtifactsResource",
    "ArtifactsResourceWithRawResponse",
    "AsyncArtifactsResourceWithRawResponse",
    "ArtifactsResourceWithStreamingResponse",
    "AsyncArtifactsResourceWithStreamingResponse",
    "UploadSchedulesResource",
    "AsyncUploadSchedulesResource",
    "UploadSchedulesResourceWithRawResponse",
    "AsyncUploadSchedulesResourceWithRawResponse",
    "UploadSchedulesResourceWithStreamingResponse",
    "AsyncUploadSchedulesResourceWithStreamingResponse",
    "KnowledgeBasesResource",
    "AsyncKnowledgeBasesResource",
    "KnowledgeBasesResourceWithRawResponse",
    "AsyncKnowledgeBasesResourceWithRawResponse",
    "KnowledgeBasesResourceWithStreamingResponse",
    "AsyncKnowledgeBasesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, Annotated, TypeAlias

from .._utils import PropertyInfo
from .._models import BaseModel
from .shared.chunk_extra_info_schema import ChunkExtraInfoSchema
from .shared.result_schema_generation import ResultSchemaGeneration
from .shared.string_extra_info_schema import StringExtraInfoSchema
from .evaluation_datasets.flexible_chunk import FlexibleChunk
from .evaluation_datasets.flexible_message import FlexibleMessage

__all__ = [
    "ApplicationTestCaseOutput",
    "ApplicationTestCaseGenerationOutputResponse",
    "ApplicationTestCaseFlexibleOutputResponse",
    "ApplicationTestCaseFlexibleOutputResponseOutput",
    "ApplicationTestCaseFlexibleOutputResponseOutputGenerationExtraInfo",
]


class ApplicationTestCaseGenerationOutputResponse(BaseModel):
    id: str
    """The unique identifier of the entity."""

    account_id: str
    """The ID of the account that owns the given entity."""

    application_variant_id: str

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    evaluation_dataset_id: str

    output: ResultSchemaGeneration

    test_case_id: str

    application_interaction_id: Optional[str] = None

    metrics: Optional[Dict[str, float]] = None

    schema_type: Optional[Literal["GENERATION"]] = None


ApplicationTestCaseFlexibleOutputResponseOutputGenerationExtraInfo: TypeAlias = Annotated[
    Union[ChunkExtraInfoSchema, StringExtraInfoSchema], PropertyInfo(discriminator="schema_type")
]


class ApplicationTestCaseFlexibleOutputResponseOutput(BaseModel):
    generation_output: Union[
        str,
        float,
        List[FlexibleChunk],
        List[FlexibleMessage],
        List[object],
        Dict[str, Union[str, float, List[FlexibleChunk], List[FlexibleMessage], List[object], object]],
        object,
    ]

    generation_extra_info: Optional[ApplicationTestCaseFlexibleOutputResponseOutputGenerationExtraInfo] = None


class ApplicationTestCaseFlexibleOutputResponse(BaseModel):
    id: str
    """The unique identifier of the entity."""

    account_id: str
    """The ID of the account that owns the given entity."""

    application_variant_id: str

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    evaluation_dataset_id: str

    output: ApplicationTestCaseFlexibleOutputResponseOutput

    test_case_id: str

    application_interaction_id: Optional[str] = None

    metrics: Optional[Dict[str, float]] = None

    schema_type: Optional[Literal["FLEXIBLE"]] = None


ApplicationTestCaseOutput: TypeAlias = Annotated[
    Union[ApplicationTestCaseGenerationOutputResponse, ApplicationTestCaseFlexibleOutputResponse],
    PropertyInfo(discriminator="schema_type"),
]



### autogenerated_evaluation_dataset_creation_example_script.py ###
# This script demonstrates how to create a autogenerated evaluation dataset
# For autogenerated evaluation datasets, a generation job workflow is created to generate test cases.
# Test cases are generated based on the knowledge base provided, and must be approved before the dataset can be published.
# Evaluation datasets, once published, can be used for application variant runs and report card generation
# For the purposes of this script, we assume that you already have a knowledge base set up with artifacts uploaded
# This can be either done through API, or by going to https://egp.dashboard.scale.com/knowledge-bases/new
import os
import time

from scale_gp import SGPClient

account_id = os.environ.get("SGP_ACCOUNT_ID", None)
api_key = os.environ.get("SGP_API_KEY", None)
knowledge_base_id = os.environ.get("SGP_KNOWLEDGE_BASE_ID", None)

assert (
    account_id is not None
), "You need to set the SGP_ACCOUNT_ID - you can find it at https://egp.dashboard.scale.com/admin/accounts"
assert api_key is not None, "You need to provide your API key - see https://egp.dashboard.scale.com/admin/api-key"

# Create an SGP Client
client = SGPClient(api_key=api_key)

### autogenerated evaluation dataset ###

# we assume we have a knowledge base with artifacts already uploaded
autogenerated_evaluation_dataset = client.evaluation_datasets.create(
    account_id=account_id,
    name="autogenerated_eval_dataset",
    schema_type="GENERATION",
    type="autogenerated",
    knowledge_base_id=knowledge_base_id,
)

# view evaluation dataset details. This will show the dataset_id, name, schema_type, type, harms_list, advanced_config etc.
dataset = client.evaluation_datasets.retrieve(evaluation_dataset_id=autogenerated_evaluation_dataset.id)

# start generation job to generate test cases
generation_job = client.evaluation_datasets.generation_jobs.create(
    evaluation_dataset_id=autogenerated_evaluation_dataset.id,
    num_test_cases=3,
    group_by_artifact_id=False,
)

while True:
    if generation_job.status == "Pending":
        print("generating test cases...")
        time.sleep(5)
    else:
        break

# view autogenerated test cases
test_cases = client.evaluation_datasets.autogenerated_draft_test_cases.list(
    evaluation_dataset_id=autogenerated_evaluation_dataset.id
)

# approve autogenerated test cases. all test cases must be approved before the dataset can be published.
for test_case in test_cases.items:
    client.evaluation_datasets.autogenerated_draft_test_cases.approve(
        evaluation_dataset_id=autogenerated_evaluation_dataset.id,
        autogenerated_draft_test_case_id=test_case.id,
    )

# snapshot into a dataset version when test cases are approved
autogenerated_dataset_version = client.evaluation_datasets.evaluation_dataset_versions.create(
    evaluation_dataset_id=autogenerated_evaluation_dataset.id
)

# publish the dataset. datasets must be published before they can be used in evaluations.
published_dataset_response = client.evaluation_datasets.publish(
    evaluation_dataset_id=autogenerated_evaluation_dataset.id,
)

print("Published autogenerated evaluation dataset with id:", autogenerated_evaluation_dataset.id)
print(autogenerated_dataset_version)

test_cases = client.evaluation_datasets.test_cases.list(
    evaluation_dataset_id=autogenerated_evaluation_dataset.id, account_id=account_id
)

print("Created the following test cases:")
print(test_cases.items)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, TypeAlias, TypedDict

__all__ = [
    "DeploymentPackageInstallAsyncParams",
    "VersionConstraint",
    "VersionConstraintSemVerMinVersion",
    "VersionConstraintTimestampMinVersion",
    "VersionConstraintExactVersion",
]


class DeploymentPackageInstallAsyncParams(TypedDict, total=False):
    deployment_package_id: str

    deployment_package_name: str

    deployment_package_type: Literal[
        "EGP_CONFIGURATION",
        "BLOB_STORAGE_FILE",
        "DOCKER_IMAGE",
        "EGP_ENTITY",
        "EGP_USER",
        "INTERNAL_MODEL_INFO",
        "EVALUATION_DATASET",
        "MODEL",
        "KNOWLEDGE_BASE",
    ]
    """An enumeration."""

    version_constraint: VersionConstraint


class VersionConstraintSemVerMinVersion(TypedDict, total=False):
    minimum_version: Required[str]

    version_constraint_type: Literal["SEMVER_MIN"]


class VersionConstraintTimestampMinVersion(TypedDict, total=False):
    minimum_timestamp: Required[str]

    version_constraint_type: Literal["TIMESTAMP_MIN"]


class VersionConstraintExactVersion(TypedDict, total=False):
    package_version_id: Required[str]

    version_constraint_type: Literal["EXACT_VERSION"]


VersionConstraint: TypeAlias = Union[
    VersionConstraintSemVerMinVersion, VersionConstraintTimestampMinVersion, VersionConstraintExactVersion
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["AutogeneratedDraftTestCaseApproveParams"]


class AutogeneratedDraftTestCaseApproveParams(TypedDict, total=False):
    evaluation_dataset_id: Required[str]

    force: bool
    """Force approve a failing test case"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.


from .._models import BaseModel

__all__ = ["CreateKnowledgeBaseResponse"]


class CreateKnowledgeBaseResponse(BaseModel):
    knowledge_base_id: str
    """The unique ID of the created knowledge base"""



### flexible_evaluation_example_script.py ###
# This script demonstrates how to create a flexible dataset and evaluation

# What is flexible dataset?

# A flexible dataset is a dataset that can be used to evaluate an application's performance against a set of test cases.
# Flexible datasets are able to perform anything a regular GENERATION dataset can do (covered by evaluation_example_script.py),
# but support a larger range of datatypes for both test case input and outputs.

# Flexible evaluation is only available for external/offline applications

# Schema for test case data:
# Input: Str | Dict[str, str | int | float | chunks | messages | list[any] | Dict[any, any]
# Expected_output: same as input but optional

# What is an evaluation?

# Evaluate an application variant against a rubric (question set) to determine how well the application variant performs.
# Depending on the evaluation type, either use a LLM, human, or mixed (LLM Benchmark) to answer questions comparing the
# response of an application variant to an expected response (test case expected output). Flexible evaluation support human evaluations only currently
# The results of the evaluation can be used to help generate a report card for the application variant, benchmark performance, and compare variants.

# Required steps to perform a flexible evaluation:
# 1. Create a flexible evaluation dataset
# 2. Insert test cases into the flexible evaluation dataset and snapshot into a dataset version.
# 3. Create at question set with at least one question. The question set will be used to evaluate the the responses to the test cases.
# 4. Create an evaluation config using the question set and a desired evaluation type (human, llm or hybrid)
# 5. Create an offline application spec and variant.
# 6. Use the ExternalApplication class and a generator function which calls your offline application to generate outputs
# 7. Create an annotation config. This helps customize the annotation UI per question.
# 8. Create an evaluation using the evaluation config, and the desired offline application variant.

import os
import time
from typing import Any, Dict

from scale_gp import SGPClient
from scale_gp.lib.external_applications import ExternalApplication, ExternalApplicationOutputCompletion
from scale_gp.types.evaluation_datasets.test_case import TestCase

account_id = os.environ.get("SGP_ACCOUNT_ID", None)
api_key = os.environ.get("SGP_API_KEY", None)

# Either set these environment variables, or a default application variant and spec will be created
external_application_variant = os.environ.get("SGP_EXTERNAL_APPLICATION_VARIANT_ID", None)
external_application_spec = os.environ.get("SGP_EXTERNAL_APPLICATION_SPEC_ID", None)

assert (
    account_id is not None
), "You need to set the SGP_ACCOUNT_ID - you can find it at https://egp.dashboard.scale.com/admin/accounts"
assert api_key is not None, "You need to provide your API key - see https://egp.dashboard.scale.com/admin/api-key"

# Create an SGP Client
client = SGPClient(api_key=api_key)

# 1. Create a flexible evaluation dataset
flexible_evaluation_dataset = client.evaluation_datasets.create(
    account_id=account_id,
    name="flexible_evaluation_dataset",
    schema_type="FLEXIBLE",
    type="manual",
)

# 2. Batch insert test cases into the flexible evaluation dataset and create a snapshot
# Below are the currently supported types for test case input and output
#     "example_bool": False,
#     "example_float": 3.14,
#     "example_int": 23,
#     "example_messages": [
#         {"role": "user", "content": "What is the capital of France?"},
#         {"role": "assistant", "content": "The capital of France is Paris."},
#         {"role": "assistant", "content": "The capital of France is Paris."},
#         {"role": "user", "content": "Thank you!"},
#         {"role": "user", "content": "Thank you!"},
#     ],
#     "example_dictionary": {
#         "title": "The Great Gatsby",
#         "author": "F. Scott Fitzgerald",
#         "published_year": 1925,
#         "genres": ["Tragedy", "Historical"],
#         "characters": {"protagonist": "Jay Gatsby", "narrator": "Nick Carraway"},
#     },
#     "example_string_array": ["sun", "moon", "stars", "comet"],
#     "example_mixed_array": [42, "banana", 3.14, True, {"name": "Alice"}],
#     "example_chunks": [
#         {
#             "text": "I have a dream that one day this nation will rise up and live out the true meaning of its creed.",
#             "metadata": {"speaker": "Martin Luther King Jr.", "year": 1963},
#         },
#         {
#             "text": "Ask not what your country can do for you – ask what you can do for your country.",
#             "metadata": {"speaker": "John F. Kennedy", "year": 1961},
#         },
#         {"text": "The only thing we have to fear is fear itself."},
#     ],
#     "example_document": """**Lorem** ipsum dolor sit amet, consectetur adipiscing elit. Donec ac feugiat metus. Morbi imperdiet eget ante vel interdum. Vestibulum finibus dolor a congue luctus. Aenean hendrerit nec orci ac pulvinar. Mauris feugiat arcu nec placerat vestibulum. Phasellus at diam gravida, rhoncus elit et, rutrum nunc. Pellentesque fermentum iaculis ligula ac rhoncus. Duis pulvinar dolor id hendrerit placerat. Sed vel suscipit sem, ac finibus tortor. Suspendisse commodo, quam sed molestie condimentum, tellus lorem suscipit mi, a fermentum dolor felis in tellus. Nullam diam felis, aliquet ac sagittis in, mattis sed sapien. Duis tempor urna eget aliquet vehicula. Suspendisse nec fermentum erat, sed molestie magna. Nullam laoreet sit amet est sed maximus. Nam eu blandit magna, ut pulvinar quam. Cras dictum eros sed mattis scelerisque.
# Cras mi mauris, pellentesque sit amet pharetra ut, viverra sed dui. Nullam elementum placerat scelerisque. Duis a libero ac purus faucibus dictum id vitae ex. Interdum et malesuada fames ac ante ipsum primis in faucibus. Suspendisse quis egestas odio. Nam non ornare augue, id ullamcorper orci. Praesent faucibus eget lectus sed sodales. Donec sed urna eget odio ornare consequat.
# **Praesent** suscipit massa a justo tempus suscipit. Aenean cursus nunc eget accumsan lacinia. Duis sit amet erat euismod, euismod est vitae, rhoncus sem. Donec aliquam quam facilisis leo accumsan, in tincidunt erat molestie. Cras accumsan convallis erat, id volutpat augue consectetur vel. Proin quis tincidunt tellus. Proin lobortis velit vel arcu commodo hendrerit. Phasellus imperdiet elementum nisl, a rutrum metus facilisis id. Cras placerat arcu sem, a porta orci luctus vitae. Aenean commodo aliquam lectus, vel dignissim augue rutrum ac. Nam pharetra at sem sit amet ultricies. Fusce quis metus et ex euismod malesuada.
# Aenean quis augue a ex cursus pretium. Aenean quis arcu tortor. Phasellus nunc nunc, pulvinar non ornare sed, imperdiet sit amet ipsum. Curabitur commodo maximus metus. Donec imperdiet, felis nec faucibus aliquet, urna nulla interdum enim, vel aliquam nunc lectus condimentum ex. Integer ac tempor mi. Nullam vel convallis massa. Vestibulum cursus odio euismod arcu varius finibus. Vestibulum in diam sit amet ex vehicula pharetra. Morbi sed orci risus. Duis venenatis pellentesque ante id ultricies. Praesent eleifend tempor tristique. Suspendisse tristique gravida metus vel rhoncus.
# **Suspendisse** ut felis gravida, molestie tellus quis, ornare enim. Aliquam bibendum sapien purus, eget posuere diam varius sed. Donec bibendum porta orci, sed scelerisque elit. Pellentesque habitant morbi tristique senectus et netus et malesuada fames ac turpis egestas. In augue turpis, tempor sed gravida a, consequat sit amet urna. Vivamus vitae iaculis nibh, in dapibus lacus. Suspendisse interdum tortor enim, in ultrices enim mollis eget. Morbi nisl massa, hendrerit nec velit pretium, vulputate vulputate nunc. Donec vitae tellus et enim sagittis dapibus. Vestibulum consequat semper diam, vitae gravida arcu semper et. Vestibulum id efficitur justo. Suspendisse a suscipit purus. Curabitur turpis quam, vestibulum et posuere eget, gravida sed dui. Sed tellus dolor, dignissim eu odio vitae, gravida facilisis risus. Pellentesque accumsan mi ante, sed finibus ligula accumsan eget. Etiam auctor mauris lectus.""",

TEST_CASES = [
    {
        "input": "Who is the first president of the united states?",
        "expected_output": "George Washington",
    },
    {
        "input": 1000000,
        "expected_output": "The number of dollars in a megadollar",
    },
    {
        "input": {
            "question_type": "who",
            "question": "Who is the second president of the united states?",
        },
        "expected_output": {
            "percieved_mode": "declaratory",
            "answer": "John Adams",
        },
    },
    {
        "input": [1, 2, 3, 4],
        "expected_output": [
            {"number": 1, "type": "integer"},
            {"number": 2, "type": "integer"},
            {"number": 3, "type": "integer"},
            {"number": 4, "type": "integer"},
        ],
    },
]

uploaded_test_cases = client.evaluation_datasets.test_cases.batch(
    evaluation_dataset_id=flexible_evaluation_dataset.id,
    items=[
        {"account_id": account_id, "schema_type": "FLEXIBLE", "test_case_data": test_case} for test_case in TEST_CASES
    ],
)
test_case_ids = [test_case.id for test_case in uploaded_test_cases]

# snapshot into a dataset version
flexible_dataset_version = client.evaluation_datasets.evaluation_dataset_versions.create(
    evaluation_dataset_id=flexible_evaluation_dataset.id
)

# 3. Create at question set with at least one question. The question set will be used to evaluate the the responses to the test cases.
questions = [
    {
        "type": "categorical",
        "title": "Test Question",
        "prompt": "Test Prompt",
        "choices": [{"label": "No", "value": 0}, {"label": "Yes", "value": 1}],
        "account_id": account_id,
    },
    {
        "type": "free_text",
        "title": "Question only about the document",
        "prompt": "What is the document about?",
        "account_id": account_id,
    },
]

question_ids = []
for question in questions:
    created_question = client.questions.create(**question)
    question_ids.append(created_question.id)

# create a question set using the desired questions
question_set = client.question_sets.create(
    account_id=account_id,
    name="US Presidents Question Set",
    question_ids=question_ids,
)

# 4. Create an evaluation config using the question set and a desired evaluation type (human only right now)
evaluation_config = client.evaluation_configs.create(
    account_id=account_id,
    evaluation_type="human",
    question_set_id=question_set.id,
)

# 5. Create an offline application spec and variant if no application spec id and variant id are provided
if not external_application_spec:
    application_spec = client.application_specs.create(
        account_id=account_id,
        description="Test application spec",
        name="test-application-spec" + str(time.time()),
    )
    external_application_spec = application_spec.id

if not external_application_variant:
    application_variant = client.application_variants.create(
        account_id=account_id,
        application_spec_id=external_application_spec,
        name="test offline application variant",
        version="OFFLINE",
        description="Test application variant",
        configuration={},
    )
    external_application_variant = application_variant.id

print("Created external application variant with id: ", external_application_variant)


# 6. Use the ExternalApplication class and a generator function which calls your offline application to generate outputs
def application(prompt: Dict[str, Any], test_case: TestCase) -> ExternalApplicationOutputCompletion:
    response = (
        "application response"  # Call your application with test case input here (test_case.test_case_data.input)
    )
    print("prompt: ", prompt)
    print("test_case: ", test_case)
    print("generated output: ", response)
    metrics = {"accuracy": 0.9}  # whatever metrics you want to track
    traces = [  # whatever traces you want to track, such as what the output of each node inside the application was
        {
            "node_id": "completion",
            "start_timestamp": "123",
            "operation_input": "test_input",
            "operation_output": "test_output",
            "duration_ms": 1000,
        },
        {
            "node_id": "retrieval",
            "start_timestamp": "123",
            "operation_input": "test_input",
            "operation_output": "test_output",
            "duration_ms": 1000,
        },
    ]
    return ExternalApplicationOutputCompletion(generation_output=response, metrics=metrics, traces=traces)


external_application = ExternalApplication(client).initialize(
    application_variant_id=external_application_variant,
    application=application,
)

print("Generating test case outputs:")
external_application.generate_outputs(
    evaluation_dataset_id=flexible_evaluation_dataset.id,
    evaluation_dataset_version=flexible_dataset_version.num,
)

generated_test_case_outputs = client.application_test_case_outputs.list(
    account_id=account_id,
    application_variant_id=external_application_variant,
    evaluation_dataset_id=flexible_evaluation_dataset.id,
    evaluation_dataset_version_num=flexible_dataset_version.num,
)

# 7. Create an annotation config. This helps customize the annotation UI per question.
# - components are a 2D layout of data (row major - can switch to column major with direction: col)
# - max width of a row are 2 items side by side
# - max number of columns are 2 when using the `direction = 'col'` layout
annotation_config_dict = {
    "annotation_config_type": "flexible",
    "direction": "row",  # this is by default - try switching to "col"
    "components": [
        [
            {
                "data_loc": ["test_case_data", "input"],
            },
            {
                "data_loc": ["test_case_data", "expected_output"],
            },
        ],
        [
            {
                "data_loc": ["test_case_output", "output"],
            },
        ],
    ],
}

# try clicking on the second question to see the annotation UI change
question_id_to_annotation_config_dict = {
    question_ids[1]: {
        "annotation_config": "flexible",  # NOTE: this has to be a flexible annotation config
        "components": [
            [
                {
                    "data_loc": ["test_case_data", "input"],
                }
            ]
        ],
    }
}

# 8. Create an evaluation using the evaluation config, and the desired offline application variant.
# In this case, we are creating a human evaluation using the evaluation config created in step 4.
# Since this is a human evaluation, human annotators will be asked the questions defined in the question set about
# the test case outputs that we batch inserted in step 6. The results will be used to determine the performance of the application
# and generate repord cards and metrics

print("Creating evaluation with flexible evaluation dataset")
evaluation = client.evaluations.create(
    type="builder",
    account_id=account_id,
    application_spec_id=external_application_spec,
    application_variant_id=external_application_variant,
    description="description",
    evaluation_dataset_id=flexible_evaluation_dataset.id,
    annotation_config=annotation_config_dict,
    question_id_to_annotation_config=question_id_to_annotation_config_dict,
    name="Flexible eval",
    evaluation_config_id=evaluation_config.id,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from scale_gp import SGPClient, AsyncSGPClient
from tests.utils import assert_matches_type
from scale_gp.types import GenericModelResponse
from scale_gp.pagination import SyncPageResponse, AsyncPageResponse
from scale_gp.types.models import (
    ModelDeployment,
)
from scale_gp.types.shared import GenericDeleteResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestDeployments:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: SGPClient) -> None:
        deployment = client.models.deployments.create(
            model_instance_id="model_instance_id",
            name="name",
        )
        assert_matches_type(ModelDeployment, deployment, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: SGPClient) -> None:
        deployment = client.models.deployments.create(
            model_instance_id="model_instance_id",
            name="name",
            account_id="account_id",
            deployment_metadata={},
            model_creation_parameters={},
            vendor_configuration={
                "max_workers": 0,
                "min_workers": 0,
                "per_worker": 0,
                "vendor": "LAUNCH",
            },
        )
        assert_matches_type(ModelDeployment, deployment, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: SGPClient) -> None:
        response = client.models.deployments.with_raw_response.create(
            model_instance_id="model_instance_id",
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        deployment = response.parse()
        assert_matches_type(ModelDeployment, deployment, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: SGPClient) -> None:
        with client.models.deployments.with_streaming_response.create(
            model_instance_id="model_instance_id",
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            deployment = response.parse()
            assert_matches_type(ModelDeployment, deployment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_create(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_instance_id` but received ''"):
            client.models.deployments.with_raw_response.create(
                model_instance_id="",
                name="name",
            )

    @parametrize
    def test_method_retrieve(self, client: SGPClient) -> None:
        deployment = client.models.deployments.retrieve(
            deployment_id="deployment_id",
            model_instance_id="model_instance_id",
        )
        assert_matches_type(ModelDeployment, deployment, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: SGPClient) -> None:
        response = client.models.deployments.with_raw_response.retrieve(
            deployment_id="deployment_id",
            model_instance_id="model_instance_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        deployment = response.parse()
        assert_matches_type(ModelDeployment, deployment, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: SGPClient) -> None:
        with client.models.deployments.with_streaming_response.retrieve(
            deployment_id="deployment_id",
            model_instance_id="model_instance_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            deployment = response.parse()
            assert_matches_type(ModelDeployment, deployment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_instance_id` but received ''"):
            client.models.deployments.with_raw_response.retrieve(
                deployment_id="deployment_id",
                model_instance_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `deployment_id` but received ''"):
            client.models.deployments.with_raw_response.retrieve(
                deployment_id="",
                model_instance_id="model_instance_id",
            )

    @parametrize
    def test_method_update(self, client: SGPClient) -> None:
        deployment = client.models.deployments.update(
            deployment_id="deployment_id",
            model_instance_id="model_instance_id",
        )
        assert_matches_type(ModelDeployment, deployment, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: SGPClient) -> None:
        deployment = client.models.deployments.update(
            deployment_id="deployment_id",
            model_instance_id="model_instance_id",
            deployment_metadata={},
            model_creation_parameters={},
            name="name",
            vendor_configuration={
                "max_workers": 0,
                "min_workers": 0,
                "per_worker": 0,
                "vendor": "LAUNCH",
            },
        )
        assert_matches_type(ModelDeployment, deployment, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: SGPClient) -> None:
        response = client.models.deployments.with_raw_response.update(
            deployment_id="deployment_id",
            model_instance_id="model_instance_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        deployment = response.parse()
        assert_matches_type(ModelDeployment, deployment, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: SGPClient) -> None:
        with client.models.deployments.with_streaming_response.update(
            deployment_id="deployment_id",
            model_instance_id="model_instance_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            deployment = response.parse()
            assert_matches_type(ModelDeployment, deployment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_instance_id` but received ''"):
            client.models.deployments.with_raw_response.update(
                deployment_id="deployment_id",
                model_instance_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `deployment_id` but received ''"):
            client.models.deployments.with_raw_response.update(
                deployment_id="",
                model_instance_id="model_instance_id",
            )

    @parametrize
    def test_method_list(self, client: SGPClient) -> None:
        deployment = client.models.deployments.list(
            model_instance_id="model_instance_id",
        )
        assert_matches_type(SyncPageResponse[ModelDeployment], deployment, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: SGPClient) -> None:
        deployment = client.models.deployments.list(
            model_instance_id="model_instance_id",
            account_id="account_id",
            limit=1,
            page=1,
            sort_by=["model_creation_parameters:asc", "model_creation_parameters:desc", "model_endpoint_id:asc"],
        )
        assert_matches_type(SyncPageResponse[ModelDeployment], deployment, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: SGPClient) -> None:
        response = client.models.deployments.with_raw_response.list(
            model_instance_id="model_instance_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        deployment = response.parse()
        assert_matches_type(SyncPageResponse[ModelDeployment], deployment, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: SGPClient) -> None:
        with client.models.deployments.with_streaming_response.list(
            model_instance_id="model_instance_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            deployment = response.parse()
            assert_matches_type(SyncPageResponse[ModelDeployment], deployment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_instance_id` but received ''"):
            client.models.deployments.with_raw_response.list(
                model_instance_id="",
            )

    @parametrize
    def test_method_delete(self, client: SGPClient) -> None:
        deployment = client.models.deployments.delete(
            deployment_id="deployment_id",
            model_instance_id="model_instance_id",
        )
        assert_matches_type(GenericDeleteResponse, deployment, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: SGPClient) -> None:
        response = client.models.deployments.with_raw_response.delete(
            deployment_id="deployment_id",
            model_instance_id="model_instance_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        deployment = response.parse()
        assert_matches_type(GenericDeleteResponse, deployment, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: SGPClient) -> None:
        with client.models.deployments.with_streaming_response.delete(
            deployment_id="deployment_id",
            model_instance_id="model_instance_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            deployment = response.parse()
            assert_matches_type(GenericDeleteResponse, deployment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_instance_id` but received ''"):
            client.models.deployments.with_raw_response.delete(
                deployment_id="deployment_id",
                model_instance_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `deployment_id` but received ''"):
            client.models.deployments.with_raw_response.delete(
                deployment_id="",
                model_instance_id="model_instance_id",
            )

    @parametrize
    def test_method_execute(self, client: SGPClient) -> None:
        deployment = client.models.deployments.execute(
            model_deployment_id="model_deployment_id",
            model_instance_id="model_instance_id",
        )
        assert_matches_type(GenericModelResponse, deployment, path=["response"])

    @parametrize
    def test_method_execute_with_all_params(self, client: SGPClient) -> None:
        deployment = client.models.deployments.execute(
            model_deployment_id="model_deployment_id",
            model_instance_id="model_instance_id",
            stream=True,
        )
        assert_matches_type(GenericModelResponse, deployment, path=["response"])

    @parametrize
    def test_raw_response_execute(self, client: SGPClient) -> None:
        response = client.models.deployments.with_raw_response.execute(
            model_deployment_id="model_deployment_id",
            model_instance_id="model_instance_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        deployment = response.parse()
        assert_matches_type(GenericModelResponse, deployment, path=["response"])

    @parametrize
    def test_streaming_response_execute(self, client: SGPClient) -> None:
        with client.models.deployments.with_streaming_response.execute(
            model_deployment_id="model_deployment_id",
            model_instance_id="model_instance_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            deployment = response.parse()
            assert_matches_type(GenericModelResponse, deployment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_execute(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_instance_id` but received ''"):
            client.models.deployments.with_raw_response.execute(
                model_deployment_id="model_deployment_id",
                model_instance_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_deployment_id` but received ''"):
            client.models.deployments.with_raw_response.execute(
                model_deployment_id="",
                model_instance_id="model_instance_id",
            )

    @parametrize
    def test_method_list_all(self, client: SGPClient) -> None:
        deployment = client.models.deployments.list_all()
        assert_matches_type(SyncPageResponse[ModelDeployment], deployment, path=["response"])

    @parametrize
    def test_method_list_all_with_all_params(self, client: SGPClient) -> None:
        deployment = client.models.deployments.list_all(
            account_id="account_id",
            limit=1,
            page=1,
            sort_by=["model_creation_parameters:asc", "model_creation_parameters:desc", "model_endpoint_id:asc"],
        )
        assert_matches_type(SyncPageResponse[ModelDeployment], deployment, path=["response"])

    @parametrize
    def test_raw_response_list_all(self, client: SGPClient) -> None:
        response = client.models.deployments.with_raw_response.list_all()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        deployment = response.parse()
        assert_matches_type(SyncPageResponse[ModelDeployment], deployment, path=["response"])

    @parametrize
    def test_streaming_response_list_all(self, client: SGPClient) -> None:
        with client.models.deployments.with_streaming_response.list_all() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            deployment = response.parse()
            assert_matches_type(SyncPageResponse[ModelDeployment], deployment, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncDeployments:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncSGPClient) -> None:
        deployment = await async_client.models.deployments.create(
            model_instance_id="model_instance_id",
            name="name",
        )
        assert_matches_type(ModelDeployment, deployment, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncSGPClient) -> None:
        deployment = await async_client.models.deployments.create(
            model_instance_id="model_instance_id",
            name="name",
            account_id="account_id",
            deployment_metadata={},
            model_creation_parameters={},
            vendor_configuration={
                "max_workers": 0,
                "min_workers": 0,
                "per_worker": 0,
                "vendor": "LAUNCH",
            },
        )
        assert_matches_type(ModelDeployment, deployment, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.models.deployments.with_raw_response.create(
            model_instance_id="model_instance_id",
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        deployment = await response.parse()
        assert_matches_type(ModelDeployment, deployment, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncSGPClient) -> None:
        async with async_client.models.deployments.with_streaming_response.create(
            model_instance_id="model_instance_id",
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            deployment = await response.parse()
            assert_matches_type(ModelDeployment, deployment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_create(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_instance_id` but received ''"):
            await async_client.models.deployments.with_raw_response.create(
                model_instance_id="",
                name="name",
            )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncSGPClient) -> None:
        deployment = await async_client.models.deployments.retrieve(
            deployment_id="deployment_id",
            model_instance_id="model_instance_id",
        )
        assert_matches_type(ModelDeployment, deployment, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.models.deployments.with_raw_response.retrieve(
            deployment_id="deployment_id",
            model_instance_id="model_instance_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        deployment = await response.parse()
        assert_matches_type(ModelDeployment, deployment, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        async with async_client.models.deployments.with_streaming_response.retrieve(
            deployment_id="deployment_id",
            model_instance_id="model_instance_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            deployment = await response.parse()
            assert_matches_type(ModelDeployment, deployment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_instance_id` but received ''"):
            await async_client.models.deployments.with_raw_response.retrieve(
                deployment_id="deployment_id",
                model_instance_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `deployment_id` but received ''"):
            await async_client.models.deployments.with_raw_response.retrieve(
                deployment_id="",
                model_instance_id="model_instance_id",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncSGPClient) -> None:
        deployment = await async_client.models.deployments.update(
            deployment_id="deployment_id",
            model_instance_id="model_instance_id",
        )
        assert_matches_type(ModelDeployment, deployment, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncSGPClient) -> None:
        deployment = await async_client.models.deployments.update(
            deployment_id="deployment_id",
            model_instance_id="model_instance_id",
            deployment_metadata={},
            model_creation_parameters={},
            name="name",
            vendor_configuration={
                "max_workers": 0,
                "min_workers": 0,
                "per_worker": 0,
                "vendor": "LAUNCH",
            },
        )
        assert_matches_type(ModelDeployment, deployment, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.models.deployments.with_raw_response.update(
            deployment_id="deployment_id",
            model_instance_id="model_instance_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        deployment = await response.parse()
        assert_matches_type(ModelDeployment, deployment, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncSGPClient) -> None:
        async with async_client.models.deployments.with_streaming_response.update(
            deployment_id="deployment_id",
            model_instance_id="model_instance_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            deployment = await response.parse()
            assert_matches_type(ModelDeployment, deployment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_instance_id` but received ''"):
            await async_client.models.deployments.with_raw_response.update(
                deployment_id="deployment_id",
                model_instance_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `deployment_id` but received ''"):
            await async_client.models.deployments.with_raw_response.update(
                deployment_id="",
                model_instance_id="model_instance_id",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncSGPClient) -> None:
        deployment = await async_client.models.deployments.list(
            model_instance_id="model_instance_id",
        )
        assert_matches_type(AsyncPageResponse[ModelDeployment], deployment, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncSGPClient) -> None:
        deployment = await async_client.models.deployments.list(
            model_instance_id="model_instance_id",
            account_id="account_id",
            limit=1,
            page=1,
            sort_by=["model_creation_parameters:asc", "model_creation_parameters:desc", "model_endpoint_id:asc"],
        )
        assert_matches_type(AsyncPageResponse[ModelDeployment], deployment, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.models.deployments.with_raw_response.list(
            model_instance_id="model_instance_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        deployment = await response.parse()
        assert_matches_type(AsyncPageResponse[ModelDeployment], deployment, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncSGPClient) -> None:
        async with async_client.models.deployments.with_streaming_response.list(
            model_instance_id="model_instance_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            deployment = await response.parse()
            assert_matches_type(AsyncPageResponse[ModelDeployment], deployment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_instance_id` but received ''"):
            await async_client.models.deployments.with_raw_response.list(
                model_instance_id="",
            )

    @parametrize
    async def test_method_delete(self, async_client: AsyncSGPClient) -> None:
        deployment = await async_client.models.deployments.delete(
            deployment_id="deployment_id",
            model_instance_id="model_instance_id",
        )
        assert_matches_type(GenericDeleteResponse, deployment, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.models.deployments.with_raw_response.delete(
            deployment_id="deployment_id",
            model_instance_id="model_instance_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        deployment = await response.parse()
        assert_matches_type(GenericDeleteResponse, deployment, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncSGPClient) -> None:
        async with async_client.models.deployments.with_streaming_response.delete(
            deployment_id="deployment_id",
            model_instance_id="model_instance_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            deployment = await response.parse()
            assert_matches_type(GenericDeleteResponse, deployment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_instance_id` but received ''"):
            await async_client.models.deployments.with_raw_response.delete(
                deployment_id="deployment_id",
                model_instance_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `deployment_id` but received ''"):
            await async_client.models.deployments.with_raw_response.delete(
                deployment_id="",
                model_instance_id="model_instance_id",
            )

    @parametrize
    async def test_method_execute(self, async_client: AsyncSGPClient) -> None:
        deployment = await async_client.models.deployments.execute(
            model_deployment_id="model_deployment_id",
            model_instance_id="model_instance_id",
        )
        assert_matches_type(GenericModelResponse, deployment, path=["response"])

    @parametrize
    async def test_method_execute_with_all_params(self, async_client: AsyncSGPClient) -> None:
        deployment = await async_client.models.deployments.execute(
            model_deployment_id="model_deployment_id",
            model_instance_id="model_instance_id",
            stream=True,
        )
        assert_matches_type(GenericModelResponse, deployment, path=["response"])

    @parametrize
    async def test_raw_response_execute(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.models.deployments.with_raw_response.execute(
            model_deployment_id="model_deployment_id",
            model_instance_id="model_instance_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        deployment = await response.parse()
        assert_matches_type(GenericModelResponse, deployment, path=["response"])

    @parametrize
    async def test_streaming_response_execute(self, async_client: AsyncSGPClient) -> None:
        async with async_client.models.deployments.with_streaming_response.execute(
            model_deployment_id="model_deployment_id",
            model_instance_id="model_instance_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            deployment = await response.parse()
            assert_matches_type(GenericModelResponse, deployment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_execute(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_instance_id` but received ''"):
            await async_client.models.deployments.with_raw_response.execute(
                model_deployment_id="model_deployment_id",
                model_instance_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_deployment_id` but received ''"):
            await async_client.models.deployments.with_raw_response.execute(
                model_deployment_id="",
                model_instance_id="model_instance_id",
            )

    @parametrize
    async def test_method_list_all(self, async_client: AsyncSGPClient) -> None:
        deployment = await async_client.models.deployments.list_all()
        assert_matches_type(AsyncPageResponse[ModelDeployment], deployment, path=["response"])

    @parametrize
    async def test_method_list_all_with_all_params(self, async_client: AsyncSGPClient) -> None:
        deployment = await async_client.models.deployments.list_all(
            account_id="account_id",
            limit=1,
            page=1,
            sort_by=["model_creation_parameters:asc", "model_creation_parameters:desc", "model_endpoint_id:asc"],
        )
        assert_matches_type(AsyncPageResponse[ModelDeployment], deployment, path=["response"])

    @parametrize
    async def test_raw_response_list_all(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.models.deployments.with_raw_response.list_all()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        deployment = await response.parse()
        assert_matches_type(AsyncPageResponse[ModelDeployment], deployment, path=["response"])

    @parametrize
    async def test_streaming_response_list_all(self, async_client: AsyncSGPClient) -> None:
        async with async_client.models.deployments.with_streaming_response.list_all() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            deployment = await response.parse()
            assert_matches_type(AsyncPageResponse[ModelDeployment], deployment, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from datetime import datetime
from typing_extensions import Literal, Annotated, TypeAlias

from ..._utils import PropertyInfo
from ..._compat import PYDANTIC_V2, ConfigDict
from ..._models import BaseModel

__all__ = [
    "ModelDeployment",
    "VendorConfiguration",
    "VendorConfigurationLaunchDeploymentVendorConfiguration",
    "VendorConfigurationLlmEngineDeploymentVendorConfiguration",
]


class VendorConfigurationLaunchDeploymentVendorConfiguration(BaseModel):
    max_workers: Optional[int] = None

    min_workers: Optional[int] = None

    per_worker: Optional[int] = None
    """The maximum number of concurrent requests that an individual worker can service.

    Launch automatically scales the number of workers for the endpoint so that each
    worker is processing `per_worker` requests, subject to the limits defined by
    `min_workers` and `max_workers`.

    - If the average number of concurrent requests per worker is lower than
      `per_worker`, then the number of workers will be reduced. - Otherwise, if the
      average number of concurrent requests per worker is higher than `per_worker`,
      then the number of workers will be increased to meet the elevated traffic.

    Here is our recommendation for computing `per_worker`:

    1. Compute `min_workers` and `max_workers` per your minimum and maximum
       throughput requirements. 2. Determine a value for the maximum number of
       concurrent requests in the workload. Divide this number by `max_workers`.
       Doing this ensures that the number of workers will "climb" to `max_workers`.
    """

    vendor: Optional[Literal["LAUNCH"]] = None


class VendorConfigurationLlmEngineDeploymentVendorConfiguration(BaseModel):
    base_model_name: Optional[str] = None

    checkpoint_path: Optional[str] = None

    cpus: Optional[int] = None

    gpu_type: Optional[
        Literal[
            "nvidia-tesla-t4",
            "nvidia-ampere-a10",
            "nvidia-ampere-a100",
            "nvidia-ampere-a100e",
            "nvidia-hopper-h100",
            "nvidia-hopper-h100-1g20gb",
            "nvidia-hopper-h100-3g40gb",
        ]
    ] = None
    """An enumeration."""

    gpus: Optional[int] = None

    high_priority: Optional[bool] = None

    inference_framework_image_tag: Optional[str] = None

    max_workers: Optional[int] = None

    memory: Optional[str] = None

    min_workers: Optional[int] = None

    model_name: Optional[str] = None

    num_shards: Optional[int] = None

    per_worker: Optional[int] = None
    """The maximum number of concurrent requests that an individual worker can service.

    Launch automatically scales the number of workers for the endpoint so that each
    worker is processing `per_worker` requests, subject to the limits defined by
    `min_workers` and `max_workers`.

    - If the average number of concurrent requests per worker is lower than
      `per_worker`, then the number of workers will be reduced. - Otherwise, if the
      average number of concurrent requests per worker is higher than `per_worker`,
      then the number of workers will be increased to meet the elevated traffic.

    Here is our recommendation for computing `per_worker`:

    1. Compute `min_workers` and `max_workers` per your minimum and maximum
       throughput requirements. 2. Determine a value for the maximum number of
       concurrent requests in the workload. Divide this number by `max_workers`.
       Doing this ensures that the number of workers will "climb" to `max_workers`.
    """

    storage: Optional[str] = None

    vendor: Optional[Literal["LLMENGINE"]] = None

    if PYDANTIC_V2:
        # allow fields with a `model_` prefix
        model_config = ConfigDict(protected_namespaces=tuple())


VendorConfiguration: TypeAlias = Annotated[
    Union[
        VendorConfigurationLaunchDeploymentVendorConfiguration,
        VendorConfigurationLlmEngineDeploymentVendorConfiguration,
    ],
    PropertyInfo(discriminator="vendor"),
]


class ModelDeployment(BaseModel):
    id: str
    """The unique identifier of the entity."""

    account_id: str
    """The ID of the account that owns the given entity."""

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    created_by_user_id: str
    """The user who originally created the entity."""

    name: str

    status: str
    """Status of the model's deployment."""

    deployment_metadata: Optional[object] = None

    model_creation_parameters: Optional[object] = None

    model_endpoint_id: Optional[str] = None

    model_instance_id: Optional[str] = None

    vendor_configuration: Optional[VendorConfiguration] = None

    if PYDANTIC_V2:
        # allow fields with a `model_` prefix
        model_config = ConfigDict(protected_namespaces=tuple())



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["LocalFileSourceConfig"]


class LocalFileSourceConfig(BaseModel):
    source: Literal["LocalFile"]

    deduplication_strategy: Optional[Literal["Overwrite", "Fail"]] = None
    """An enumeration."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, TypeAlias

from ..._models import BaseModel
from .artifact_schema_generation import ArtifactSchemaGeneration
from .generation_test_case_schema import GenerationTestCaseSchema
from ..auto_generated_draft_test_case_missing_chunk_info import AutoGeneratedDraftTestCaseMissingChunkInfo

__all__ = ["AutoGeneratedDraftTestCasesList", "Item", "ItemTestCaseData"]

ItemTestCaseData: TypeAlias = Union[ArtifactSchemaGeneration, GenerationTestCaseSchema]


class Item(BaseModel):
    id: str
    """The unique identifier of the entity."""

    account_id: str
    """The ID of the account that owns the given entity."""

    approved: bool
    """Boolean to track whether or not the draft test case is approved"""

    content_metadata: object
    """
    Metadata for the chunks and artifacts that were used to generate this draft test
    case.
    """

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    created_by_user_id: str
    """The user who originally created the entity."""

    evaluation_dataset_id: str
    """The ID of the associated evaluation dataset."""

    schema_type: Literal["GENERATION", "FLEXIBLE"]
    """An enumeration."""

    test_case_data: ItemTestCaseData
    """The data for the test case in a format matching the provided schema_type"""

    updated_at: datetime
    """The date and time when the entity was last updated in ISO format."""

    missing_chunks: Optional[List[AutoGeneratedDraftTestCaseMissingChunkInfo]] = None

    topic_str: Optional[str] = None


class AutoGeneratedDraftTestCasesList(BaseModel):
    current_page: int
    """The current page number."""

    items: List[Item]
    """The data returned for the current page."""

    items_per_page: int
    """The number of items per page."""

    total_item_count: int
    """The total number of items of the query"""
